#coding=utf-8
'''
android手机相关action设定
Created on 2017年6月13日

@author: Administrator
'''

class SettingAction(object):
    
    ACTION_SETTINGS = "android.settings.SETTINGS" #设置界面
    ACTION_APN_SETTINGS = "android.settings.APN_SETTINGS"
    ACTION_LOCATION_SOURCE_SETTINGS ="android.settings.LOCATION_SOURCE_SETTINGS"
    ACTION_USER_SETTINGS ="android.settings.USER_SETTINGS" # system
    ACTION_WIRELESS_SETTINGS ="android.settings.WIRELESS_SETTINGS"
    ACTION_TETHER_PROVISIONING ="android.settings.TETHER_PROVISIONING_UI"  #hide
    ACTION_AIRPLANE_MODE_SETTINGS ="android.settings.AIRPLANE_MODE_SETTINGS"
    ACTION_VOICE_CONTROL_AIRPLANE_MODE ="android.settings.VOICE_CONTROL_AIRPLANE_MODE"
    ACTION_ACCESSIBILITY_SETTINGS = "android.settings.ACCESSIBILITY_SETTINGS"
    ACTION_USAGE_ACCESS_SETTINGS = "android.settings.USAGE_ACCESS_SETTINGS"
    INTENT_CATEGORY_USAGE_ACCESS_CONFIG ="android.intent.category.USAGE_ACCESS_CONFIG"
    ACTION_SECURITY_SETTINGS ="android.settings.SECURITY_SETTINGS"
    ACTION_TRUSTED_CREDENTIALS_USER ="com.android.settings.TRUSTED_CREDENTIALS_USER"
    ACTION_MONITORING_CERT_INFO ="com.android.settings.MONITORING_CERT_INFO"
    ACTION_PRIVACY_SETTINGS ="android.settings.PRIVACY_SETTINGS"
    ACTION_VPN_SETTINGS ="android.settings.VPN_SETTINGS"
    ACTION_WIFI_SETTINGS ="android.settings.WIFI_SETTINGS"
    ACTION_WIFI_IP_SETTINGS ="android.settings.WIFI_IP_SETTINGS"
    ACTION_BLUETOOTH_SETTINGS ="android.settings.BLUETOOTH_SETTINGS"
    ACTION_CAST_SETTINGS ="android.settings.CAST_SETTINGS"
    ACTION_DATE_SETTINGS ="android.settings.DATE_SETTINGS"
    ACTION_SOUND_SETTINGS ="android.settings.SOUND_SETTINGS"
    ACTION_DISPLAY_SETTINGS ="android.settings.DISPLAY_SETTINGS"
    ACTION_NIGHT_DISPLAY_SETTINGS ="android.settings.NIGHT_DISPLAY_SETTINGS"
    ACTION_LOCALE_SETTINGS ="android.settings.LOCALE_SETTINGS"
    ACTION_VOICE_INPUT_SETTINGS ="android.settings.VOICE_INPUT_SETTINGS"
    ACTION_INPUT_METHOD_SETTINGS ="android.settings.INPUT_METHOD_SETTINGS"
    ACTION_INPUT_METHOD_SUBTYPE_SETTINGS ="android.settings.INPUT_METHOD_SUBTYPE_SETTINGS"
    ACTION_SHOW_INPUT_METHOD_PICKER ="android.settings.SHOW_INPUT_METHOD_PICKER"
    ACTION_USER_DICTIONARY_SETTINGS ="android.settings.USER_DICTIONARY_SETTINGS"
    ACTION_HARD_KEYBOARD_SETTINGS ="android.settings.HARD_KEYBOARD_SETTINGS"
    ACTION_USER_DICTIONARY_INSERT ="com.android.settings.USER_DICTIONARY_INSERT"
    ACTION_APPLICATION_SETTINGS ="android.settings.APPLICATION_SETTINGS"
    ACTION_APPLICATION_DEVELOPMENT_SETTINGS ="android.settings.APPLICATION_DEVELOPMENT_SETTINGS"
    ACTION_QUICK_LAUNCH_SETTINGS ="android.settings.QUICK_LAUNCH_SETTINGS"
    ACTION_MANAGE_APPLICATIONS_SETTINGS ="android.settings.MANAGE_APPLICATIONS_SETTINGS"
    ACTION_MANAGE_ALL_APPLICATIONS_SETTINGS ="android.settings.MANAGE_ALL_APPLICATIONS_SETTINGS"
    ACTION_MANAGE_OVERLAY_PERMISSION ="android.settings.action.MANAGE_OVERLAY_PERMISSION"
    ACTION_MANAGE_WRITE_SETTINGS ="android.settings.action.MANAGE_WRITE_SETTINGS"
    ACTION_APPLICATION_DETAILS_SETTINGS ="android.settings.APPLICATION_DETAILS_SETTINGS"
    ACTION_IGNORE_BATTERY_OPTIMIZATION_SETTINGS ="android.settings.IGNORE_BATTERY_OPTIMIZATION_SETTINGS"
    ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS ="android.settings.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS"
    ACTION_IGNORE_BACKGROUND_DATA_RESTRICTIONS_SETTINGS ="android.settings.IGNORE_BACKGROUND_DATA_RESTRICTIONS_SETTINGS"
    ACTION_SYSTEM_UPDATE_SETTINGS ="android.settings.SYSTEM_UPDATE_SETTINGS"
    ACTION_SYNC_SETTINGS ="android.settings.SYNC_SETTINGS"
    ACTION_ADD_ACCOUNT ="android.settings.ADD_ACCOUNT_SETTINGS"
    ACTION_NETWORK_OPERATOR_SETTINGS ="android.settings.NETWORK_OPERATOR_SETTINGS"
    ACTION_DATA_ROAMING_SETTINGS ="android.settings.DATA_ROAMING_SETTINGS"
    ACTION_INTERNAL_STORAGE_SETTINGS ="android.settings.INTERNAL_STORAGE_SETTINGS"
    ACTION_MEMORY_CARD_SETTINGS ="android.settings.MEMORY_CARD_SETTINGS"
    ACTION_SEARCH_SETTINGS ="android.search.action.SEARCH_SETTINGS"
    ACTION_DEVICE_INFO_SETTINGS ="android.settings.DEVICE_INFO_SETTINGS"
    ACTION_NFC_SETTINGS = "android.settings.NFC_SETTINGS"
    ACTION_NFCSHARING_SETTINGS ="android.settings.NFCSHARING_SETTINGS"
    ACTION_NFC_PAYMENT_SETTINGS ="android.settings.NFC_PAYMENT_SETTINGS"
    ACTION_DREAM_SETTINGS = "android.settings.DREAM_SETTINGS"
    ACTION_NOTIFICATION_LISTENER_SETTINGS = "android.settings.ACTION_NOTIFICATION_LISTENER_SETTINGS"
    ACTION_NOTIFICATION_POLICY_ACCESS_SETTINGS = "android.settings.NOTIFICATION_POLICY_ACCESS_SETTINGS"
    ACTION_CONDITION_PROVIDER_SETTINGS = "android.settings.ACTION_CONDITION_PROVIDER_SETTINGS"
    ACTION_CAPTIONING_SETTINGS = "android.settings.CAPTIONING_SETTINGS"
    ACTION_PRINT_SETTINGS ="android.settings.ACTION_PRINT_SETTINGS"
    ACTION_ZEN_MODE_SETTINGS = "android.settings.ZEN_MODE_SETTINGS"
    ACTION_ZEN_MODE_PRIORITY_SETTINGS = "android.settings.ZEN_MODE_PRIORITY_SETTINGS"
    ACTION_ZEN_MODE_AUTOMATION_SETTINGS = "android.settings.ZEN_MODE_AUTOMATION_SETTINGS"
    ACTION_VOICE_CONTROL_DO_NOT_DISTURB_MODE = "android.settings.VOICE_CONTROL_DO_NOT_DISTURB_MODE"
    ACTION_ZEN_MODE_SCHEDULE_RULE_SETTINGS = "android.settings.ZEN_MODE_SCHEDULE_RULE_SETTINGS"
    ACTION_ZEN_MODE_EVENT_RULE_SETTINGS = "android.settings.ZEN_MODE_EVENT_RULE_SETTINGS"
    ACTION_ZEN_MODE_EXTERNAL_RULE_SETTINGS = "android.settings.ZEN_MODE_EXTERNAL_RULE_SETTINGS" # hide
    ACTION_SHOW_REGULATORY_INFO = "android.settings.SHOW_REGULATORY_INFO"
    DEVICE_NAME_SETTINGS = "android.settings.DEVICE_NAME" #hide
    ACTION_PAIRING_SETTINGS = "android.settings.PAIRING_SETTINGS" #hide
    ACTION_BATTERY_SAVER_SETTINGS = "android.settings.BATTERY_SAVER_SETTINGS"
    ACTION_VOICE_CONTROL_BATTERY_SAVER_MODE = "android.settings.VOICE_CONTROL_BATTERY_SAVER_MODE"
    ACTION_HOME_SETTINGS = "android.settings.HOME_SETTINGS"
    ACTION_MANAGE_DEFAULT_APPS_SETTINGS = "android.settings.MANAGE_DEFAULT_APPS_SETTINGS"
    ACTION_NOTIFICATION_SETTINGS = "android.settings.NOTIFICATION_SETTINGS" #hide
    ACTION_APP_NOTIFICATION_SETTINGS = "android.settings.APP_NOTIFICATION_SETTINGS" #hide
    ACTION_APP_NOTIFICATION_REDACTION = "android.settings.ACTION_APP_NOTIFICATION_REDACTION" #hide
    ACTION_SHOW_ADMIN_SUPPORT_DETAILS = "android.settings.SHOW_ADMIN_SUPPORT_DETAILS" #hide
    ACTION_SHOW_REMOTE_BUGREPORT_DIALOG = "android.settings.SHOW_REMOTE_BUGREPORT_DIALOG" #hide
    ACTION_VR_LISTENER_SETTINGS = "android.settings.VR_LISTENER_SETTINGS"
    ACTION_STORAGE_MANAGER_SETTINGS = "android.settings.STORAGE_MANAGER_SETTINGS"
    ACTION_WEBVIEW_SETTINGS = "android.settings.WEBVIEW_SETTINGS"
    

        


# coding=utf-8
"""
Created on 2015年11月4日

@author: thomas.ning
"""

import itertools,os,socket
import utils,time,json,requests
from requests.exceptions import ReadTimeout
from easyuiautomator.driver.common.errorCode import ErrorCode
from easyuiautomator.common.log import Logger
from easyuiautomator.driver.common.mobilecommand import MobileCommand
import base64
import threading
import subprocess

UIAUTOMATOR_ROOT_PATH = "/data/local/tmp/"
CONFIG_FILE = os.path.join(UIAUTOMATOR_ROOT_PATH, "yepbootstrap.json")
UIAUTOMATOR_PORT = 4724
LOCAL_PORT = 4724
sdk_target_version=21 # statup u1 or u2 condition
u2_version_code = 4
u2_package_name = "com.testguard.uiautomator2"


_init_local_port = LOCAL_PORT - 1

# get auto assign port
def next_local_port(adbHost=None):
    def is_port_listening(port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = s.connect_ex((str(adbHost) if adbHost else '127.0.0.1', port))
        s.close()
        return result == 0
    global _init_local_port
    _init_local_port = _init_local_port + 1 if _init_local_port < 32764 else LOCAL_PORT
    while is_port_listening(_init_local_port):
        _init_local_port += 1
    return _init_local_port


class Uiautomator(object):
    """
    Uiautomator Used to start stop service And complete command interaction
    """
    __U1 = ["YepTelecomBootstrap.jar","resource/u1/YepTelecomBootstrap.jar"]
    __U2 = ["resource/u2/testguard-uiautomator.apk", "resource/u2/testguard-uiautomator-test.apk"]

    def __init__(self, adb, sdk_version, local_port=None):
        self.logger = Logger.getLogger()
        self._adb = adb
        self.__sdk = sdk_target_version
        self.sdk_version = sdk_version
        self._uiauprocess = None
        self.version_code = 0
        self.resource_path = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
        self.logger.debug('start init uiautomator on %s'%self._adb.serial)
        self.local_port = local_port if local_port else self._getConnectedPort()
        self.logger.debug("clear server config file on %s"%self._adb.serial)
        self._adb.delFile(CONFIG_FILE)
        self.timeout = 15 # http访问超时值
        self.restart_num = 3 # 本地服务重启次数
        self.http_post = HttpPost(self.rpc_uri, self.timeout)
       
    @property
    def rpc_uri(self):
        return "http://%s:%d/" % (self._adb.server_host, self.local_port)
    
    def push(self):
        filename = os.path.join(self.resource_path, self.__U1[1])
        self._adb.push(filename, UIAUTOMATOR_ROOT_PATH)
    
    def install(self):
        for apk in self.__U2:
            self._adb.installrt(os.path.join(self.resource_path, apk))
            
    def removePkg(self):
        self._adb.uninstallApk(u2_package_name,True)
        self._adb.uninstallApk(u2_package_name +".test")
    
    def start(self, timeout=20, restart=False):
        self.start_new(timeout, restart)
    
    def start_new(self, timeout=30, restart=False):
        '''新的启动方式，加入预先ping的方案'''
        if self.sdk_version < self.__sdk:
            cmd = " ".join(list(itertools.chain(
                    ["shell", "uiautomator", "runtest"],
                    [self.__U1[0]],
                    ["-c", "io.appium.android.bootstrap.Bootstrap"],
                    ["--nohup"]
                ))) 
        else:
            cmd = " ".join(
                        ["shell", "am", "instrument", "-w",
                       "com.testguard.uiautomator2.test/android.support.test.runner.AndroidJUnitRunner&"])
        if not restart:
            if self.sdk_version < self.__sdk:
                self.push()
            else:
                if self.checkVersion():
                    self.removePkg()
                    self.install() 
        self._adb.forward(self.local_port, UIAUTOMATOR_PORT)
        if self.pre_ping(3):
            return 
        self.stop()
        self._adb.forward(self.local_port, UIAUTOMATOR_PORT)
        if restart:
            self.logger.debug('Restart Uiautomator Server on %s'%self._adb.serial)
        else:
            self.logger.debug('Startup Uiautomator Server on %s'%self._adb.serial)
        self._uiauprocess = self._adb.cmd(cmd)
        time.sleep(3)
        self.waitServer(timeout)
        
    def checkVersion(self):
        '''检查uiautomator apk版本  if true remove'''
        if self.version_code == 0:
            self.version_code = self._adb.getVersionCode(u2_package_name)
        return True if u2_version_code > self.version_code else False

    def waitServer(self, timeout):
        # connect u2 server
        while not self.alive and timeout > 0:
            time.sleep(0.1)
            timeout -= 0.1
        if not self.alive:
            if self.restart_num == 0:
                self.restart_num = 3
                raise IOError(utils.formatExceptionStr(self._adb.serial, "本地服务无法启动"))
            else:
                self.restart_num -= 1
                self.logger.debug("restart device server on %s,total=3,restarting=%s"%(self._adb.serial,3-self.restart_num))
                self.start(restart=True) 
        else:
            self.restart_num = 3

    @property
    def alive(self):
        '''Check if the rpc server is alive.'''
        try:
            data = self.ping()
            if data is not None:
                return data['status']==0
            return False
        except:
            return False
        
    def pre_ping(self, timeout):
        '''预ping,如果已民连接，则正常执行'''
        while True: 
            if self.alive is True:
                return True
            if timeout > 0:
                time.sleep(1)
                timeout -= 1
            else: 
                return False
        
    def ping(self):
        try:
            extra = {"action": MobileCommand.PING, "params": {}}
            return self.http_post.post(extra, 5) 
        except:
            return None
        
    def sendActon(self, action, params):
        extra = {"action": action, "params": params}
        try:
            return self.http_post.post(extra)
        except ReadTimeout:
            raise
        except:
            if params and params.get("elementId"):
                raise
            return self._restartServer(extra)
              
    def _restartServer(self, data=None):
        self.start(restart=True)
        if data:
            return self.http_post.post(data)

    def _getConnectedPort(self):
        '''auto assign port'''
        local_port = UIAUTOMATOR_PORT
        try:  # first we will try to use the local port already adb forwarded
            for s, lp, rp in self._adb.forwardlist():
                if s == self._adb.serial and rp == 'tcp:%d' % UIAUTOMATOR_PORT:
                    local_port = int(lp[4:])
                    break
            else:
                local_port = next_local_port()
        except:
            local_port = next_local_port()
        return local_port

    
    def stopUiauto(self):
        if self._uiauprocess and self._uiauprocess.poll() is None:
            if self._uiauprocess:
                self._uiauprocess.terminate()
            self._uiauprocess = None
        self._adb.killProcessesByName("uiautomator")
        self._adb.forceStop(u2_package_name)

    def stop(self):
        self.stop_uri()
        self.stopUiauto()
            
    def stop_uri(self):
        stop_url = "http://%s:%d/stop" % (self._adb.server_host, self.local_port)
        try:
            requests.post(stop_url,timeout=5)
        except:
            pass

def systemCmd(cmd_line):
    '''exec system cmd, paramas list'''
    if os.name != "nt":
        cmd_line = [" ".join(cmd_line)]
    return subprocess.Popen(cmd_line, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
           
class HttpPost:
    
    headers={"Content-Type": "application/json"}
    
    def __init__(self, url, timeout):
        requests.adapters.DEFAULT_RETRIES = 3
        self.url = url
        self.timeout = timeout
        
    def post(self,data,timeout=None):
        # add mintor timeout
        t = threading.Timer(90, self.stopUiautomator, (self.url,))
        t.setDaemon(True)
        t.start()
        try:
            return self._post(data, timeout)
        except:
            raise
        finally:
            try:
                t.cancel()
            except:
                pass
        
    def stopUiautomator(self, url):
        port = url.split(":")[2].split("/")[0]
        serial = None
        try:
            lines = systemCmd(['adb','forward','--list']).communicate()[0].decode("utf-8").strip().splitlines()
            for s, lp, rp in [line.strip().split() for line in lines]:
                if lp == 'tcp:%s'%port and rp=='tcp:4724':
                    serial = s
                    break
        except:
            pass
        if serial:
            os.system("adb -s %s shell am force-stop com.testguard.uiautomator2"%serial)

    def _post(self, data, timeout=None):
#         print data
        if data['action'] != 'takeScreenshot':
            if data['action'] in [MobileCommand.LONG_CLICK,MobileCommand.SCROLL_TO,"element:scrollOrientation"]:
                r = requests.post(self.url, data=json.dumps(data), headers=self.headers)
                return json.loads(r.content)
            else:
                r = requests.post(self.url, data=json.dumps(data), headers=self.headers, timeout=self.timeout if timeout is None else timeout)
                return json.loads(r.content)
        else:
            r = requests.post(self.url, data=json.dumps(data), headers=self.headers, timeout=30, stream=True)
            img = ''
            for chunk in r.iter_content(chunk_size=1024):  
                if chunk: # filter out keep-alive new chunks  
                    img += chunk
            if len(img) > 50:
                return {'status': 0,'value': base64.b64encode(img)}
            else:
                return {'status': 1,'value': 'screenshot failure'}


#coding=utf-8
try:
    import cv2
except:
    pass

#   rmBlackBorder: remove the black borders of one image
#   return: cropped image
def _rmBlackBorder(
    src,  # input image
    thres=0,  # threshold for cropping: sum([r,g,b] - [0,0,0](black))
    diff=1000,  # max tolerable difference between black borders on two side
    shrink=0,  # number of pixels to shrink after the blackBorders removed
    directionMore=0,  # 哪个方向上多出不对称的内容，0：正常，1：上，2：下,3：左，4：右
    ):
    
    #
    #   remove the black border on both right and left side
    #
    srcCol = src.shape[1]
    srcRow = src.shape[0]
    nRow = src.shape[0]
    nCol = src.shape[1]
    left = -1
    right = nCol

    for i in [0, nRow / 2, nRow - 1]:
        curLeft = -1
        curRight = nCol
        for j in range(0, nCol - 1):
            if(sum(list(src[i, j])) <= thres and curLeft == j - 1):
                curLeft += 1
        if left == -1:
            left = curLeft
        if curLeft < left:
            left = curLeft

        for j in range(nCol - 1, 0, -1):
            if(sum(list(src[i, j])) <= thres and curRight == j + 1):
                curRight -= 1
        if right == nCol:
            right = curRight
        if curRight > right:
            right = curRight
    
    if min(left, right) >= 1 and abs((left + 1) - (nCol - right)) <= diff and right - left > 0 :
        print 'left margin: %d\n' % left
        print 'right margin: %d\n' % right
        
        if directionMore == 3:
            left = srcCol - right                    
        elif directionMore == 4:
            right = srcCol - left
        
        src = src[0 : nRow - 1, left + 1 + shrink : right - 1 - shrink, :]
    else:
        src = src

    #
    #   remove the black border on both up and down side
    #
    nRow = src.shape[0]
    nCol = src.shape[1]
    up = -1
    down = nRow    
    for j in [0, nCol / 2, nCol - 1]:
        curUp = -1
        curDown = nRow
        for i in range(0, nRow - 1):
            if(sum(list(src[i, j])) <= thres and curUp == i - 1):
                curUp += 1
        if up == -1:
            up = curUp
        if curUp < up:
            up = curUp
        for i in range(nRow - 1, 0, -1):
            if(sum(list(src[i, j])) <= thres and curDown == i + 1):
                curDown -= 1
        if down == nRow:
            down = curDown
        if curDown > down:
            down = curDown    
    if min(up, down) >= 1 and abs((up + 1) - (nRow - down)) <= diff and down - up > 0:
        if directionMore == 4 or directionMore == 1:
            up = srcRow - down
        elif directionMore == 3 or directionMore == 2:
            down = srcRow - up         
        print 'up margin: %d\n' % up
        print 'down margin: %d\n' % down
        dst = src[up + 1 + shrink : down - 1 - shrink, 0 : nCol - 1, :]
    else:
        dst = src
    return dst


def blackBorderCheck(
            srcpath,  # input image
            tarpath,
            thres,  # threshold for cropping: sum([r,g,b] - [0,0,0](black))
            diff,  # max tolerable difference between black borders on two side
            shrink,  # number of pixels to shrink after the blackBorders removed
            directionMore,  # 哪个方向上多出不对称的内容，0：正常，1：上，2：下,3：左，4：右
            ):

    src = cv2.imread(srcpath)    
    res = _rmBlackBorder(src,thres, diff, shrink, directionMore)
    cv2.imwrite(tarpath, res)




if __name__ == '__main__':
    path = r"C:\Program Files\AutoRecorder\workspace\test_tool\tools\test.png"
    path2 = r"C:\Program Files\AutoRecorder\workspace\test_tool\tools\test2.png"
    print blackBorderCheck(path, path2, 50,1000,0,0)




#coding=utf-8
'''
Created on 2017年3月27日

@author: Administrator
'''
import unittest
from easyuiautomator.driver.driver import Driver
Driver.set_debug()
app = Driver.connect_device()

class TestElement(unittest.TestCase):


    def testAttribute(self):
        ele = app.find_element_by_accessibility_id("联系人")
        print "text: " + ele.getText()
        print "resourceId: " + ele.getResourceId()
        print "classname: " + ele.getClassName()
        print "package: "
        print "context-desc: "
        print "checkable: " + str(ele.isCheckable())
        print "Checked: " + str(ele.isChecked())
        print "clickable: " + str(ele.isClickable())
        print "enabled: " + str(ele.isEnabled())
        print "focuable: " + str(ele.isFocuable())
        print "focused: " + str(ele.isFocused())
        print "scrollable: " + str(ele.isScrollable())
        print "longclickable: " + str(ele.isLongClickable())
        print "seleceted : " + str(ele.isSelected())
        print "diaplayed: " + str(ele.isDisplayed())
        print "location: " + str(ele.getLocation())
        print "size: " + str(ele.getSize())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


#coding=utf-8
try:
    import cv2
    import numpy as np
except:
    pass

FLANN_INDEX_KDTREE = 1  # bug: flann enums are missing
FLANN_INDEX_LSH    = 6

def _middlePoint(pts):
    '''get center positon by given points'''
    def add(p1, p2):
        return (p1[0]+p2[0], p1[1]+p2[1])
    def distance(p1, p2):
        import math
        l2 = (p1[0]-p2[0])**2 + (p1[1]-p2[1])**2
        return math.sqrt(l2)
    length = len(pts) # point set length
    sumx, sumy = reduce(add, pts)
    point = sumx/length, sumy/length
    # filter out ok points
    avg_distance = sum([distance(point, p) for p in pts])/length
    good = []
    sumx, sumy = 0.0, 0.0
    for p in pts:
#         print 'point: %s, distance: %.2f' %(p, distance(p, point))
        if distance(p, point) < 1.2*avg_distance:
            good.append(p)
            sumx += p[0]
            sumy += p[1]
        else:
            pass
#             print 'not good', p
    point = map(long, (sumx/len(good), sumy/len(good)))
    return point

def init_feature(name):
    '''choice algorithm'''
    chunks = name.split('-')
    if chunks[0] == 'sift':
        detector = cv2.SIFT()
        norm = cv2.NORM_L2
    elif chunks[0] == 'surf':
        detector = cv2.SURF(800)
        norm = cv2.NORM_L2
    elif chunks[0] == 'orb':
        detector = cv2.ORB(400)
        norm = cv2.NORM_HAMMING
    else:
        return None, None
    if 'flann' in chunks:
        if norm == cv2.NORM_L2:
            flann_params = dict(algorithm = FLANN_INDEX_KDTREE, trees = 5)
        else:
            flann_params= dict(algorithm = FLANN_INDEX_LSH,
                               table_number = 6, # 12
                               key_size = 12,     # 20
                               multi_probe_level = 1) #2
        matcher = cv2.FlannBasedMatcher(flann_params, {})  # bug : need to pass empty dict (#1329)
    else:
        matcher = cv2.BFMatcher(norm)
    return detector, matcher


def filter_matches(kp1, kp2, matches, ratio = 0.75):
    '''过滤匹配点'''
    mkp1, mkp2 = [], []
    for m in matches:
        if len(m) == 2 and m[0].distance < m[1].distance * ratio:
            m = m[0]
            mkp1.append( kp1[m.queryIdx] )
            mkp2.append( kp2[m.trainIdx] )
    p1 = np.float32([kp.pt for kp in mkp1])
    p2 = np.float32([kp.pt for kp in mkp2])
    return p1, p2

def _find_position(img1, H = None):
    '''find img position'''
    h1, w1 = img1.shape[:2]
    corners = np.float32([[0, 0], [w1, 0], [w1, h1], [0, h1]])
    corners = np.int32( cv2.perspectiveTransform(corners.reshape(1, -1, 2), H).reshape(-1, 2))
#     arr = corners < 0
#     if True in arr:
#         return None
    return _middlePoint(corners)

def find_img_position(query, origin, algorithm='sift',radio=0.75):
    '''
      return position of query in origin,by ratio and algorithm
    :Args:
        - origin - raw picture 
        - query -  need search picture
        - ratio -  similarity 
        - algorithm - using algorithm
    :Usage:
      find_img_position('query.png','qq.png','sift', 0.75)  
    '''
    img1 = cv2.imread(query, 0)
    img2 = cv2.imread(origin, 0)
    detector, matcher = init_feature(algorithm)
    kp1, desc1 = detector.detectAndCompute(img1, None)
    kp2, desc2 = detector.detectAndCompute(img2, None)
    raw_matches = matcher.knnMatch(desc1, trainDescriptors = desc2, k = 2) #2
    p1, p2 = filter_matches(kp1, kp2, raw_matches)
    if len(p1) >= 4:
        H, status = cv2.findHomography(p1, p2, cv2.RANSAC, 5.0)
        if round(np.sum(status)/float(len(status)),2) < radio:
            return None
        return _find_position(img1, H)
    return None

if __name__ == '__main__':
    print find_img_position("child.png", "big.png", algorithm="sift",radio=0.6)




# coding=utf-8
"""
Created on 2015年11月5日

@author: thomas.ning

"""

"""
Exceptions that may happen in all the driver code.

"""

class DriverException(Exception):
    
    """
    Base driver exception.
    """
    pass

        
class NoSuchElementException(DriverException):
    """
    Thrown when element could not be found.

    If you encounter this exception, you may want to check the following:
        * Check your selector used in your find_by...
        * Element may not yet be on the screen at the time of the find operation,
        (webpage is still loading) see selenium.webdriver.support.wait.WebDriverWait() 
        for how to write a wait wrapper to wait for an element to appear.
    """
    pass


class InvalidSelectorException(NoSuchElementException):
    """
    Thrown when the selector which is used to find an element does not return
    a WebElement. Currently this only happens when the selector is an xpath
    expression and it is either syntactically invalid (i.e. it is not a
    xpath expression) or the expression does not select WebElements
    (e.g. "count(//input)").
    """
    pass


class TimeoutException(DriverException):
    """
    Thrown when a command does not complete in enough time.
    """
    pass


class SocketConnectException(DriverException):
    """
    Thrown when socket does not connect
    """

    pass


class AdbException(DriverException):
    '''
    Thrown when error occured in ADB.
    '''
    pass

class DeviceNotFound(AdbException):
    '''
    Thrown when error occured in ADB.
    '''
    pass


class UiautomatorException(DriverException):
    '''
    Throm when uiautumator return fault value.
    '''
    pass

class RemoteServiceException(DriverException):
    '''
    Throm when resource server return fault value.
    '''
    pass

class FileNotFound(DriverException):
    '''
    Throm when file not exist.
    '''
    pass


# coding=utf-8
'''
Created on 2015年11月9日

@author: thomas.ning
'''

import time
from easyuiautomator.common.exceptions import NoSuchElementException
from easyuiautomator.common.exceptions import TimeoutException

POLL_FREQUENCY = 0.5  # How long to sleep inbetween calls to the method
IGNORED_EXCEPTIONS = (NoSuchElementException,)  # exceptions ignored during calls to the method


class DriverWait(object):
    def __init__(self, driver, timeout=2, poll_frequency=POLL_FREQUENCY, ignored_exceptions=None):
        """Constructor, takes a WebDriver instance and timeout in seconds.

           :Args:
            - driver - Instance of WebDriver (Ie, Firefox, Chrome or Remote)
            - timeout - Number of seconds before timing out
            - poll_frequency - sleep interval between calls
              By default, it is 0.5 second.
            - ignored_exceptions - iterable structure of exception classes ignored during calls.
              By default, it contains NoSuchElementException only.

           Example:
            from selenium.webdriver.support.ui import WebDriverWait \n
            element = WebDriverWait(driver, 10).until(lambda x: x.find_element_by_id("someId")) \n
            is_disappeared = WebDriverWait(driver, 30, 1, (ElementNotVisibleException)).\ \n
                        until_not(lambda x: x.find_element_by_id("someId").is_displayed())
        """
        self._driver = driver
        self._timeout = timeout
        self._poll = poll_frequency
        # avoid the divide by zero
        if self._poll == 0:
            self._poll = POLL_FREQUENCY
        exceptions = list(IGNORED_EXCEPTIONS)
        if ignored_exceptions is not None:
            try:
                exceptions.extend(iter(ignored_exceptions))
            except TypeError:  # ignored_exceptions is not iterable
                exceptions.append(ignored_exceptions)
        self._ignored_exceptions = tuple(exceptions)

    def __repr__(self):
        return '<{0.__module__}.{0.__name__} (session="{1}")>'.format(
            type(self), self._driver.session_id)

    def until(self, method, message=''):
        """Calls the method provided with the driver as an argument until the \
        return value is not False."""
        screen = None
        stacktrace = None

        end_time = time.time() + self._timeout
        while True:
            try:
                value = method(self._driver)
                if value:
                    return value
            except self._ignored_exceptions as exc:
                screen = getattr(exc, 'screen', None)
                stacktrace = getattr(exc, 'stacktrace', None)
            time.sleep(self._poll)
            if time.time() > end_time:
                break
        raise TimeoutException(message, screen, stacktrace)

    def until_not(self, method, message=''):
        """Calls the method provided with the driver as an argument until the \
        return value is False."""
        end_time = time.time() + self._timeout
        while True:
            try:
                value = method(self._driver)
                if not value:
                    return value
            except self._ignored_exceptions:
                return True
            time.sleep(self._poll)
            if time.time() > end_time:
                break
        raise TimeoutException(message)
        


# coding=utf-8
'''
Created on 2015年11月11日

@author: fengbo
'''

class KeyCode(object):
    '''
    KEYCODE列表
    '''
    KEYCODE_CALL=5    # 拨号键
    KEYCODE_ENDCALL=6   # 挂机键    
    KEYCODE_HOME=3   # 按键Home    
    KEYCODE_MENU=82  # 菜单键    
    KEYCODE_BACK=4   # 返回键    
    KEYCODE_SEARCH=84   # 搜索键    
    KEYCODE_CAMERA=27   # 拍照键    
    KEYCODE_FOCUS=80    # 拍照对焦键    
    KEYCODE_POWER=26    # 电源键    
    KEYCODE_NOTIFICATION=83   # 通知键    
    KEYCODE_MUTE=91     # 话筒静音键    
    KEYCODE_VOLUME_MUTE=163   # 扬声器静音键    
    KEYCODE_VOLUME_UP=24      # 音量增加键    
    KEYCODE_VOLUME_DOWN=25    # 音量减小键    

    KEYCODE_ENTER=66    # 回车键    
    KEYCODE_ESCAPE=111  # ESC键    
    KEYCODE_DPAD_CENTER=23    # 导航键 确定键    
    KEYCODE_DPAD_UP=19        # 导航键 向上    
    KEYCODE_DPAD_DOWN=20      # 导航键 向下    
    KEYCODE_DPAD_LEFT=21      # 导航键 向左    
    KEYCODE_DPAD_RIGHT=22     # 导航键 向右    
    KEYCODE_MOVE_HOME=122     # 光标移动到开始键    
    KEYCODE_MOVE_END=123      # 光标移动到末尾键    
    KEYCODE_PAGE_UP=92        # 向上翻页键    
    KEYCODE_PAGE_DOWN=93      # 向下翻页键    
    KEYCODE_DEL=67      # 退格键    
    KEYCODE_FORWARD_DEL=112   # 删除键    
    KEYCODE_INSERT=124   # 插入键    
    KEYCODE_TAB=61   # Tab键    
    KEYCODE_NUM_LOCK=143   # 小键盘锁    
    KEYCODE_CAPS_LOCK=115   # 大写锁定键    
    KEYCODE_BREAK=121   # Break/Pause键    
    KEYCODE_SCROLL_LOCK=116  #  滚动锁定键    
    KEYCODE_ZOOM_IN=168  #  放大键    
    KEYCODE_ZOOM_OUT=169   # 缩小键    
    
    KEYCODE_0=7    # 按键'0'    
    KEYCODE_1=8    # 按键'1'    
    KEYCODE_2=9    # 按键'2'
    KEYCODE_3=10   # 按键'3'    
    KEYCODE_4=11   # 按键'4'    
    KEYCODE_5=12   # 按键'5'    
    KEYCODE_6=13   # 按键'6'    
    KEYCODE_7=14   # 按键'7'    
    KEYCODE_8=15   # 按键'8'    
    KEYCODE_9=16   # 按键'9'    
    KEYCODE_A=29   # 按键'A'    
    KEYCODE_B=30   # 按键'B'    
    KEYCODE_C=31   # 按键'C'    
    KEYCODE_D=32   # 按键'D'    
    KEYCODE_E=33   # 按键'E'    
    KEYCODE_F=34   # 按键'F'    
    KEYCODE_G=35   # 按键'G'    
    KEYCODE_H=36   # 按键'H'    
    KEYCODE_I=37   # 按键'I'    
    KEYCODE_J=38   # 按键'J'    
    KEYCODE_K=39   # 按键'K'    
    KEYCODE_L=40   # 按键'L'    
    KEYCODE_M=41   # 按键'M'    
    KEYCODE_N=42   # 按键'N'    
    KEYCODE_O=43   # 按键'O'    
    KEYCODE_P=44   # 按键'P'
    KEYCODE_Q=45   # 按键'Q'    
    KEYCODE_R=46   # 按键'R'    
    KEYCODE_S=47   # 按键'S'    
    KEYCODE_T=48   # 按键'T'    
    KEYCODE_U=49   # 按键'U'    
    KEYCODE_V=50   # 按键'V'    
    KEYCODE_W=51   # 按键'W'    
    KEYCODE_X=52   # 按键'X'    
    KEYCODE_Y=53   # 按键'Y'    
    KEYCODE_Z=54   # 按键'Z'    


# coding=utf-8
'''
Created on 2015年11月5日

@author: thomas.ning
'''
import base64, time, sys, json
from easyuiautomator.driver.common.by import By
from easyuiautomator.common.exceptions import TimeoutException
from easyuiautomator.common.exceptions import InvalidSelectorException
from easyuiautomator.common.exceptions import DeviceNotFound
from easyuiautomator.driver.common.mobilecommand import MobileCommand as Command
from easyuiautomator.driver.executor.command_executor import CommandExecutor
from easyuiautomator.driver.executor.wait import DriverWait
from easyuiautomator.driver.executor.utils import ImageUtil
from easyuiautomator.driver.executor.adb import ADB
from easyuiautomator.common.log import Logger
from easyuiautomator.driver.common.touch_action import TouchAction
from easyuiautomator.driver.common.multi_action import MultiAction
from easyuiautomator.driver.element import Element
from easyuiautomator.driver.common.keyevent import KeyCode
from easyuiautomator.common.view_remote_resource import ViewRemoteResource
import threading


# 收集异常信息
def collectException(func):
    def wrap(*args, **argkw):
        try:
            return func(*args, **argkw)
        except DeviceNotFound, e:
            raise DeviceNotFound(e)
        except:
            msg = sys.exc_info()[1].message
            Logger.getLogger().warning("%s is failed [%s]" % (func.__name__, msg))  
            if 'ignoreExp' not in  argkw.keys():
                raise
            if not argkw['ignoreExp']:
                raise  
    return wrap

class Driver(object):
    '''对外提供所有操作接口，统一采用此类输出'''
    
    __deviceMmonitor = []
    
    def __init__(self, device_id=None, port=None, restart=True):
        self.command_executor = CommandExecutor(device_id, port, restart)
        self.adb = self.command_executor._adb
        self.serial = self.adb.serial

    @staticmethod
    def connect_device(device_id=None, port=None, restart=True):
        return Driver(device_id, port, restart)

    @staticmethod
    def set_debug(whether=True):
        '''打开调试日志
        :Args:
            whether:True or False
        :Usage:
            Driver.set_debug()
        '''
        Logger._debug = whether

    @staticmethod
    def set_thinkTime(thinkTime=0.2):
        '''设置全局动作时间，单位秒
        :Args:
            thinkTime
        :Usage:
            Driver.set_thinkTime(2)
        '''
        CommandExecutor._thinkTime = thinkTime
    
    # Timeouts
    def set_timeout(self, time_to_wait):
        """
        Sets a sticky timeout to wait for an element to be found,or a command to complete.
        This method only needs to be called one time per driver
        :Args:
         - time_to_wait: Amount of time to wait (in seconds)
        :Usage:
            driver.set_timeout(30)
        """
        self._execute(Command.SET_TIMEOUT, {'ms': float(time_to_wait)})
    
    @staticmethod
    def addListen(fn):
        Driver.__deviceMmonitor.append(fn)
        
    @staticmethod
    def device_monitor(msg):
        for fn in Driver.__deviceMmonitor:
            if hasattr(fn, '__call__'):
                try:
                    fn(msg)
                except:
                    pass

    def find_element_by_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds an element by id.
        :Args:
         - id\_ - The id of the element to be found.
        :Usage:
            driver.find_element_by_id('foo')
        """
        return self.find_element(by=By.ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds multiple elements by id.
        :Args:
         - id\_ - The id of the elements to be found.
        :Usage:
            driver.find_elements_by_id('foo')
        """
        return self.find_elements(by=By.ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_xpath(self, xpath, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds an element by xpath.
        :Args:
         - xpath - The xpath locator of the element to find.
        :Usage:
            driver.find_element_by_xpath('//div/td[1]')
        """
        return self.find_element(by=By.XPATH, value=xpath, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_xpath(self, xpath, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds multiple elements by xpath.
        :Args:
         - xpath - The xpath locator of the elements to be found.
        :Usage:
            driver.find_elements_by_xpath("//div[contains(@class, 'foo')]")
        """
        return self.find_elements(by=By.XPATH, value=xpath, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds an element by name.
        :Args:
         - name: The name of the element to find.
        :Usage:
            driver.find_element_by_name('foo')
        """
        return self.find_element(by=By.NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds elements by name.
        :Args:
         - name: The name of the elements to find.

        :Usage:
            driver.find_elements_by_name('foo')
        """
        return self.find_elements(by=By.NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_class_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds an element by class name.
        :Args:
         - name: The class name of the element to find.
        :Usage:
            driver.find_element_by_class_name('foo')
        """
        return self.find_element(by=By.CLASS_NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_class_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds elements by class name.
        :Args:
         - name: The class name of the elements to find.
        :Usage:
            driver.find_elements_by_class_name('foo')
        """
        return self.find_elements(by=By.CLASS_NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_uiautomator(self, uia_string, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds element by uiautomator in Android.
        :Args:
         - uia_string - The element name in the Android UIAutomator library

        :Usage:
            driver.find_element_by_android_uiautomator('.elements()[1].cells()[2]')
        """
        return self.find_element(by=By.ANDROID_UIAUTOMATOR, value=uia_string, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_uiautomator(self, uia_string, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds elements by uiautomator in Android.
        :Args:
         - uia_string - The element name in the Android UIAutomator library

        :Usage:
            driver.find_elements_by_android_uiautomator('.elements()[1].cells()[2]')
        """
        return self.find_elements(by=By.ANDROID_UIAUTOMATOR, value=uia_string, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_accessibility_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds an element by accessibility id.
        :Args:
         - id - a string corresponding to a recursive element search using the
         Id/Name that the native Accessibility options utilize

        :Usage:
            driver.find_element_by_accessibility_id()
        """
        return self.find_element(by=By.ACCESSIBILITY_ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_accessibility_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds elements by accessibility id.

        :Args:
         - id - a string corresponding to a recursive element search using the
         Id/Name that the native Accessibility options utilize

        :Usage:
            driver.find_elements_by_accessibility_id()
        """
        return self.find_elements(by=By.ACCESSIBILITY_ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def create_element(self, element_id):
        """
        Creates a web element with the specified element_id.
        """
        return Element(self, element_id)

    def _wrap_value(self, value):
        if isinstance(value, dict):
            converted = {}
            for key, val in value.items():
                converted[key] = self._wrap_value(val)
            return converted
        elif isinstance(value, Element):
            return {'ELEMENT': value.id}
        elif isinstance(value, list):
            return list(self._wrap_value(item) for item in value)
        else:
            return value

    def _unwrap_value(self, value):
        if isinstance(value, dict) and ('ELEMENT' in value):
            wrapped_id = value.get('ELEMENT', None)
            if wrapped_id:
                return self.create_element(value['ELEMENT'])
        elif isinstance(value, list):
            return list(self._unwrap_value(item) for item in value)
        else:
            return value

    def _execute(self, driver_command, params=None):
        """
        Sends a command to be executed by a command.CommandExecutor.
        :Args:
         - driver_command: The name of the command to execute as a string.
         - params: A dictionary of named parameters to send with the command.
        :Returns:
          The command's JSON response loaded into a dictionary object.  
        """
        result = self.command_executor.execute(driver_command, params)
        # 检查消息体是否满足格式要求
        if result is not None:
            result['value'] = self._unwrap_value(result.get('value', None))
            return result
        return {'success': 0, 'value': None}
    
    @collectException
    def find_element(self, by=By.ID, value=None, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        'Private' method used by the find_element_by_* methods.
        :Usage:
            Use the corresponding find_element_by_* instead of this.
        :rtype: Element
        """
        if not By.is_valid(by) or not isinstance(value, str):
            raise InvalidSelectorException("Invalid locator values passed in")
        if thinkTime is not None :
            thinkTime = float(thinkTime)
        if timeOut is not None:
            timeOut = float(timeOut)
        return self._execute(Command.FIND_ELEMENT,
                             {'using': by, 'value': value, 'thinkTime':thinkTime, 'timeOut':timeOut})['value']
    @collectException
    def find_elements(self, by=By.ID, value=None, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        'Private' method used by the find_element_by_* methods.
        :Usage:
            Use the corresponding find_element_by_* instead of this.

        :rtype: Element
        """
        if not By.is_valid(by) or not isinstance(value, str):
            raise InvalidSelectorException("Invalid locator values passed in")
        if thinkTime is not None :
            thinkTime = float(thinkTime)
        if timeOut is not None:
            timeOut = float(timeOut)
        return self._execute(Command.FIND_ELEMENTS,
                             {'using': by, 'value': value, 'thinkTime':thinkTime, 'timeOut':timeOut})['value']
                             
    def drag(self, start_x, start_y, end_x, end_y, duration=None):
        """Drag the (x,y) to the destination element
        :Args:
         - (x,y) - the position to drag
         - destinationEl - the element to drag to
        """
        opts = {
            "startX": start_x,
            "startY": start_y,
            'endX':end_x,
            'endY':end_y,
            "duration": duration
        }
        self._execute(Command.DRAG, opts)
        return self
    
    def click(self, x, y):
        """Click position (x,y)
        :Args:
         - (x,y) - the position to be click
        """
        opts = {
            "x": x,
            "y": y,
        }
        self._execute(Command.CLICK, opts)
        return self
    
    def longClick(self, x, y, duration=None):
        """LongClick position (x,y)
        :Args:
         - (x,y,duration) - the position to be longClick then holding for a certain time
        """
        opts = {
              'x':x,
              'y':y,
              'duration':duration
              }
        self._execute(Command.LONG_CLICK, opts)
        return self
    
    def tap(self, positions, duration=None):
        """Taps on an particular place with up to five fingers, holding for a
        certain time
        :Args:
         - positions - an array of tuples representing the x/y coordinates of
         the fingers to tap. Length can be up to five.
         - duration - (optional) length of time to tap, in s

        :Usage:
            driver.tap([(100, 20), (100, 60), (100, 100)], 1)
        """
        if len(positions) == 1:
            action = TouchAction(self)
            x = positions[0][0]
            y = positions[0][1]
            if duration:
                action.press(x=x, y=y).wait(duration).release()
            else:
                action.press(x=x, y=y).release()
            action.perform()
        else:
            ma = MultiAction(self)
            for position in positions:
                x = position[0]
                y = position[1]
                action = TouchAction(self)
                if duration:
                    action.press(x=x, y=y).wait(duration).release()
                else:
                    action.press(x=x, y=y).release()
                ma.add(action)

            ma.perform()
        return self

    def swipe(self, start_x, start_y, end_x, end_y, duration=None):
        """Swipe from one point to another point, for an optional duration.
        :Args:
         - start_x - x-coordinate at which to start
         - start_y - y-coordinate at which to start
         - end_x - x-coordinate at which to stop
         - end_y - y-coordinate at which to stop
         - duration - (optional) time to take the swipe, in ms.
        :Usage:
            driver.swipe(100, 100, 100, 400)
        """
        # `swipe` is something like press-wait-moveTo-release, which the server
        # will translate into the correct action
        if duration is None:
            duration = 0
        opts = {
               "startX": start_x,
               "startY": start_y,
               "endX": end_x,
               "endY": end_y,
               "duration":duration
               }
        self._execute(Command.SWIPE, opts)
        return self

    def flick(self, start_x, start_y, end_x, end_y):
        """Flick from one point to another point.
        :Args:
         - start_x - x-coordinate at which to start
         - start_y - y-coordinate at which to start
         - end_x - x-coordinate at which to stop
         - end_y - y-coordinate at which to stop
        :Usage:
            driver.flick(100, 100, 100, 400)
        """
        action = TouchAction(self)
        action \
            .press(x=start_x, y=start_y) \
            .moveTo(x=end_x, y=end_y) \
            .release()
        action.perform()
        return self

    def press_keycode(self, keycode, metastate=None):
        """Sends a keycode to the device. Android only. Possible keycodes can be
        found in http://developer.android.com/reference/android/view/KeyEvent.html.

        :Args:
         - keycode - the keycode to be sent to the device
         - metastate - meta information about the keycode being sent
        """
        data = {
            'keycode': keycode,
            'metastate': metastate
        }
        self._execute(Command.PRESS_KEYCODE, data)["value"]
        return self

    def longpress_keycode(self, keycode, metastate=None, duration=None):
        """
        Sends a long press of keycode to the device. Android only. Possible keycodes can be
        found in http://developer.android.com/reference/android/view/KeyEvent.html.
        :Args:
         - keycode - the keycode to be sent to the device
         - metastate - meta information about the keycode being sent
        """
        data = {
            'keycode': keycode,
            'metastate': metastate,
            'duration' : duration
        }
        self._execute(Command.LONG_PRESS_KEYCODE, data)
        return self
    
    def touch_action(self):
        '''
        collection of a group action
        :Usage:
        driver.touch_action().press(10,10).wait(1).move_to(100,100).wait(2).press(200,200).realease().perform()
        '''
        action = TouchAction(self)
        return action
    
    def multi_action(self):
        '''
        collection of a group touch_action
        :Usage:
        action1 = driver.touch_action().press(10,10).wait(1).move_to(100,100).wait(2).press(200,200)
        action2 = driver.touch_action().press(10,10).wait(1).move_to(100,100).wait(2).press(200,200)
        action3 = driver.touch_action().press(10,10).wait(1).move_to(100,100).wait(2).press(200,200)
        driver.multi_action().add(action1,action2,action3).perform()
        '''
        action = MultiAction(self)
        return action
        
    def pressMenu(self):
        self.press_keycode(KeyCode.KEYCODE_MENU)
        return self
    
    def pressHome(self):
        self.press_keycode(KeyCode.KEYCODE_HOME)
        return self
    
    def pressVolup(self, num=1):
        while num > 0:
            self.press_keycode(KeyCode.KEYCODE_VOLUME_UP)
            num = num - 1    
        return self
    
    def pressVoldown(self, num=1):
        while num > 0:
            self.press_keycode(KeyCode.KEYCODE_VOLUME_DOWN)
            num = num - 1    
        return self
    
    def pressPower(self):
        self.press_keycode(KeyCode.KEYCODE_POWER)
        return self

    def _back(self, num=1):
        while num > 0:
            self._execute(Command.PRESS_BACK)
            num = num - 1
        return self
    
    def back(self, num=1):
        def _back():
            self.adb.shell("input keyevent 4")
        while num>0:
            t = threading.Thread(target=_back)
            t.setDaemon(True)
            t.start()
            time.sleep(0.2)
            num -= 1

    def wake(self):
        self._execute(Command.WAKE)
        return self

    def compressed_layout_hierarchy(self, compressLayout=False):

        opts = {
            "compressLayout": compressLayout
        }
        self._execute(Command.COMPRESSD_LAYOUT_HIERARCHY, opts)
        return self

    def get_data_dir(self):
        '''get data dir as /data/local/tmp'''
        return self._execute(Command.GET_DATA_DIR)['value']

    def open_notification(self):
        '''Open Notification'''
        self._execute(Command.OPEN_NOTIFICATION)
        return self

    def get_device_size(self):
        '''get device size'''
        opt = {
             "flag":False
             }
        return self._execute(Command.GET_DEVICE_SIZE, opt)['value']

    def get_page_source(self):
        """
        Gets the source of the current page.
        :Usage:
            driver.page_source
        """
        return self._execute(Command.GET_PAGE_SOURCE)['value']

    def get_screenshot_as_file(self, filename, scale=1.0, quality=50):
        """
        Gets the screenshot of the current window. Returns False if there is
           any IOError, else returns True. Use full paths in your filename.
        :Args:
         - filename: The full path you wish to save your screenshot to.
         - scale: The screen size scale,default:1.0.
         - quality: The screen quality,default:50%  0-100.
        :Usage:
            driver.get_screenshot_as_file('/Screenshots/foo.png',0.6,100)
        """
        png = self.get_screenshot_as_png(scale, quality)
        try:
            with open(filename, 'wb') as f:
                f.write(png)
        except IOError:
            return False
        finally:
            del png
        return True

    def get_screenshot_as_png(self, scale=1.0, quality=50):
        """
        Gets the screenshot of the current window as a binary data.
        :Args:
         - scale: The screen size scale,default:1.0.
         - quality: The screen quality,default:50%  0-100.
        :Usage:
            driver.get_screenshot_as_png(0.8,80%)
        """
        return base64.b64decode(self.get_screenshot_as_base64(scale, quality).encode('ascii'))

    def get_screenshot_as_base64(self, scale=1.0, quality=50):
        """
        Gets the screenshot of the current window as a base64 encoded string
           which is useful in embedded images in HTML.
        :Args:
         - scale: The screen size scale,default:1.0.
         - quality: The screen quality,default:50%  0-100.
        :Usage:
            driver.get_screenshot_as_base64()
        """
        opts = {
            "scale": scale,
            "quality": quality
        }
        return self._execute(Command.SCREENSHOT, opts)['value']

    def setOrientation(self, direction):
        '''
        Set device screen orientation
        params: landscape,portrait
        :Usage:
             driver.setOrientation(LANDSCAPE) set screen direction to LANDSCAPE
        '''
        opt = {
             "orientation": direction
             }
        return self._execute(Command.ORIENTAION, opt)['value']

    def getOrientation(self):
        '''Get device screen orientation'''
        opt = {}
        return self._execute(Command.ORIENTAION, opt)['value']

    def pageLeft(self, duration=3):
        """向左翻页
        :Usage:
            driver.pageLeft()
        """
        self.swipe(0.1, 0.5, 0.9, 0.5, duration)
        return self

    def pageRight(self, duration=3):
        """向右翻页
        :Usage:
            driver.pageRight()
        """
        self.swipe(0.9, 0.5, 0.1, 0.5, duration)
        return self

    def pageUp(self, duration=3):
        """向上翻页
        :Usage:
            driver.pageUp()
        """
        self.swipe(0.5, 0.5, 0.5, 0.9, duration)
        return self

    def pageDown(self, duration=3):
        """ 向下翻页
        :Usage:
            driver.pageDown()
        """
        self.swipe(0.5, 0.5, 0.5, 0.1, duration)
        return self
    
    def open_quickSetting(self):
        '''打开快速设置'''
        self._execute(Command.OPEN_QUICK_SETTING)
        return self
    
    def open_recentApps(self):
        '''打开最近运行的app'''
        self._execute(Command.OPEN_RECENT_APPS)
        return self
    
    def freezeRotation(self):
        '''冻结方向传感器  '''
        opts = {
            "status" : True,
        }
        self._execute(Command.ROTATION, opts)
        return self
    
    def unfreezeRotation(self):
        '''解冻方向传感器  '''
        opts = {
            "status" : False,
        }
        self._execute(Command.ROTATION, opts)
        return self
    
    def setText(self, text, replace=True):
        '''输入文本内容'''
        opts = {
              "text" :text,
              "replace":replace
              }
        self._execute(Command.SET_TEXT, opts)
        
    def sendKeys(self, text, replace=True):
        '''输入文本内容'''
        opts = {
              "text" :text,
              "replace":replace
              }
        self._execute(Command.SEND_KEYS, opts)
        
    def waitForIdle(self, timeout=None):
        '''等待设备空闲，默认10s'''
        opts = {
                'timeout': timeout
                }
        self._execute(Command.WAIT_FOR_IDLE, opts)
     
    def startApp(self, package, mode=0):
        '''通过包名启动程序启动界面
        :Args:
           package: 程序包名
           mode:  模式: 0全新启动，1:直接打开，保留上一次打开状态
        '''
        opts = {
              'package':package,
              'mode':mode
              }
        self._execute(Command.START_APP, opts)
        
    def start_battery_monitor(self, high=50, low=15, adbd_status=True, adbdThreshold=20):
        '''
        Start Battery Monitor.

        :Args:
         - high - Charge to specifical percents level.
                  The default is 60 percents.
         - low - if battery level lower than this value,test will pause for a lot of time until the level up to high percents.
                 The default is 20 percents.
        - adbdThreshold adbd monitor time threshold, unit(minutes)

        :Usage:
            driver.start_battery_monitor()
        '''
        opts = {
            "power_status" : True,
            "high" : high,
            "low" : low,
            "adbd_status" : adbd_status,
            "adbdThreshold": adbdThreshold
        }
        return self._execute(Command.MONITOR, opts)

    def stop_battery_monitor(self):
        '''Stop Battery Monitor '''
        opts = {
            "power_status" : False,
            "adbd_status" : False
        }
        return self._execute(Command.MONITOR, opts)

    ################# ADB #########################

    def start_activity(self, package, activity):
        """
        Open the app activities.打开应用活动
        :Args:
         - package - package name of app.
         - activity - activity of app
        :Usage:
            driver.start_activity('net.csdn.app','net.csdn.app.mainactivity')
        """
        opts = {
            'package': package,
            'activity': activity
        }
        self._execute(Command.START_ACTIVITY, opts)
        return self

    def wait_activity(self, activity, timeout, interval=1):
        """Wait for an activity: block until target activity presents
        or time out.
        This is an Android-only method.
        :Args:
         - activity - target activity
         - timeout - max wait time, in seconds
         - interval - sleep interval between retries, in seconds
        """
        try:
            DriverWait(self, timeout, interval).until(
                lambda d: d.get_current_app()[1] == activity)
            return True
        except TimeoutException:
            return False

    def get_current_app(self):
        """Retrieves the current app on the device.获取当前应用
        :Usage:
            driver.get_current_app()
        """
        return self._execute(Command.GET_CURRENT_APP)['value']

    def hide_keyboard(self):
        """
        Hide keyboard if it exists in screen.隐藏手机软键盘
        :Usage:
            driver.hide_keyboard()
        """
        self._execute(Command.HIDE_KEYBOARD)
        return self

    def pull_file(self, srcfile, tarfile):
        """
        Put file from cellphone to PC.将文件从手机放入计算机
        :Args:
         - srcFile - source file
         - targetFile - target file
        :Usage:
            driver.pull_file('/storage/sdcard0/360Download/new.txt','D:/')
        """
        opts = {
            'srcfile': srcfile,
            'tarfile': tarfile
        }
        self._execute(Command.PULL_FILE, opts)
        return self

    def pull_folder(self, remotepath, localpath):
        """
        Put folder from cellphone to PC. 将文件夹从手机放入计算机
        :Args:
         - srcFile - source folder path
         - targetFile - target path
        :Usage:
            driver.pull_folder('/storage/sdcard0/360Download','D:/')
        """
        opts = {
            'localpath': localpath,
            'remotepath': remotepath
        }
        self._execute(Command.PULL_FOLDER, opts)
        return self

    def push_file(self, srcfile, tarfile):
        '''
        Put file or folder from PC to cellphone.将文件或者文件夹从计算机放入手机
        :Args:
         - localpath - local file path in phone
         - remotepath - path in pc
        :Usage:
            driver.push_file('D:/screen.txt','/storage/sdcard0/360Download/')
        '''
        opts = {
            'srcfile': srcfile,
            'tarfile': tarfile
        }
        self._execute(Command.PUSH_FILE, opts)
        return self

    def lock(self):
        """
        Lock the screen.
        :Usage:
            driver.lock()
        """
        self._execute(Command.LOCK)
        return self

    def reset(self, pkg):
        """
        Reset app by clear it.清理应用数据重置应用
        :Args:
         - pkg _ app package.
        :Usage:
            driver.reset('net.csdn.app')
        """
        opts = {
            'pkg': pkg
        }
        self._execute(Command.RESET, opts)
        return self

    def shake(self):
        '''
        Shake cellphone. 手机震动
        :Usage:
            driver.shake()

        :Warning:
            It will be useful in future.
        '''
        self._execute(Command.SHAKE)
        return self

    def background(self, tms):
        '''
        Back to home idle and return to app activity.返回到HOME界面并随后回到应用界面
        :Args:
         - tms - :seconds
        :Usage:
            driver.background(1)
        '''
        opts = {'tms': tms}
        self._execute(Command.BACKGROUND, opts)
        return self

    def is_app_installed(self, apk):
        '''
        Whether the app is installed or not.手机应用是否安装,第三方应用
        :Args:
         - apk - :app package
        :Usage:
            driver.is_app_installed('net.csdn.app')
        '''
        opts = {
            'apk': apk
        }
        return self._execute(Command.IS_APP_INSTALLED, opts)['value']

    def install_app(self, apk, arg=None):
        '''
        Install the app.安装手机应用
        :Args:
         - apk - file.apk
        :Usage:
            driver.install_app('d:/csdn.apk')
        '''
        opts = {
            'apk': apk,
            'arg': arg
        }
        self._execute(Command.INSTALL_APP, opts)
        return self
    
    def install_remote_app(self, apk):
        '''
        Install remote app.安装手机应用
        :Args:
         - apk - file.apk
        :Usage:
            driver.install_remote_app('/data/local/tmp/csdn.apk')
        '''
        opts = {
            'apk': apk
        }
        return self._execute(Command.INSTALL_REMOTE_APP, opts)['value']
       
    def remove_app(self, apk):
        '''
        Uninstall the app.卸载手机应用
        :Args:
         - apk - app package
        :Usage:
            remove_app('net.csdn.app')
        '''
        opts = {
            'apk': apk
        }
        self._execute(Command.REMOVE_APP, opts)
        return self

    def launch_app(self, apk, activity):
        '''
        Launch the app.启动手机应用
        :Args:
         - apk - app package
         - activity - app activity
        :Usage:
            driver.launch_app('net.csdn.app','net.csdn.app.mainactivity')
        :Warning:
            The same with start_activity for the time being
        '''
        opts = {
            'apk': apk,
            'activity': activity
        }
        self._execute(Command.LAUNCH_APP, opts)
        return self

    def close_app(self, apk):
        '''
        exit the app.退出手机应用
        :Args:
         - apk - app package name
        :Usage:
            driver.close_app('net.csdn.app')
        '''
        opts = {
            'apk': apk
        }
        self._execute(Command.CLOSE_APP, opts)
        return self

    def adb_command(self, command):
        '''
        Execute ADB command。执行ADB命令
        :Args:
          - command - ADB command
        :Usage:
            command: "adb -s " + {command}
            driver.adb_command('devices')
        '''
        opts = {
            'command': command
        }
        return self._execute(Command.DO_ADB, opts)['value']
    
    def getAdb(self, restart=True):
        ''''返回adb操作集，restart为错误时，是否重启adb'''
        return ADB(self.serial, restart)

    def sleep(self, seconds):
        '''Sleep several seconds.
        :Args:
         - seconds - seconds
        :Usage:
            driver.sleep(1)
        '''
        time.sleep(seconds)
        
    @collectException
    def clear_cache(self):
        self._execute(Command.CLEAR_CACHE)

############################################################
    def find_img_position(self, query, origin, algorithm='sift', radio=0.75):
        '''
            search picture,return position
        :Args:
         - query: Target file of need search.
         - origin: driver.get_screenshot_as_png or file path
         - algorithm: default sift
        '''
        return ImageUtil.find_image_positon(query, origin, algorithm, radio)
    
    def compare_stream(self, target_file, image_stream):
        '''
        file stream compare
        :Args:
         - strStream: strStrem by driver.get_screenshot_as_png.
         - target_file: need compared target file.
        '''
        return ImageUtil.compare_stream(image_stream, target_file)
    
    def compare(self, image1, image2):
        """
        Calculate the similarity  between f1 and f2
        return similarity  0-100
        """
        return ImageUtil.compare(image1, image2)

    def crop(self, startx, starty, endx, endy, scrfile, destfile):
        """
        cut img by the given coordinates and picture, then make target file
        """
        return ImageUtil.crop(startx, starty, endx, endy, scrfile, destfile)
    
    def quit(self):
        '''disconnect device,then release resource'''
        self.command_executor.stop()
        self.command_executor = None
    
    def reboot(self):
        '''重启手机'''
        return self.adb_command("reboot")
    
    def isOnline(self):
        '''设备是否在线'''
        return self._execute(Command.IS_ONLINE)['value']
    
    def isScreenLocked(self):
        '''屏幕是否锁着'''
        return self._execute(Command.IS_LOCKED)['value']
    
    def startToast(self):
        '''开启后台toast获取'''
        opts = {
              'flag':True
              }
        self._execute(Command.TOAST, opts)
        
    def stopToast(self):
        '''获取toast并关闭监听'''
        opts = {
              'flag':False
              }
        return self._execute(Command.TOAST, opts)['value']
    
    def call(self, num):
        '''拨打电话'''
        self.adb_command("shell am start -a android.intent.action.CALL tel:%s" % num)
    
    def sendMMS(self, to_phone, msg):
        '''发送短信'''
        self.adb_command("shell am start -a android.intent.action.SENDTO -d sms:%s --es sms_body %s" % (to_phone, msg))        
    
    def openWifi(self):
        opts = {
            'type':"set",
            'flag':True
            }
        return self._execute(Command.WIFI, opts)['value']
    
    def closeWifi(self):
        opts = {
            'type':"set",
            'flag':False
            }
        return self._execute(Command.WIFI, opts)['value']
    
    def getWifiStatus(self):
        opts = {
            'type':"get_status",
            }
        return self._execute(Command.WIFI, opts)['value']
    
    def getWifiInfo(self):
        opts = {
            'type':"get_info",
            }
        result = self._execute(Command.WIFI, opts)['value']
        if result:
            return json.loads(result)
        return result
    
    def setWifiConnect(self, hotname, password, ctype):
        opts = {
            'type':"connect",
            'hot':hotname,
            'password':password,
            'ctype':ctype
            }
        self._execute(Command.WIFI, opts)
    
    def disconnectWifi(self):
        opts = {
            'type':"disconnect",
            }
        self._execute(Command.WIFI, opts)
    
    def openBluetooth(self):
        opts = {
             'flag':True
             }
        self._execute(Command.BLUETOOTH, opts)
    
    def closeBluetooth(self):
        opts = {
             'flag':False
             }
        self._execute(Command.BLUETOOTH, opts)
    
    def openGps(self):
        opts = {
             'flag':True
             }
        self._execute(Command.GPS, opts)
    
    def closeGps(self):
        opts = {
             'flag':False
             }
        self._execute(Command.GPS, opts)
    
    def add_call_log(self, phone, num):
        opts = {
             'type':"add",
             'phone':phone,
             'num': num,
             }
        self._execute(Command.CALL_LOG, opts)
    
    def clear_call_log(self):
        opts = {
             'type':"clear"
             }
        self._execute(Command.CALL_LOG, opts)
    
    def add_contact(self, phone, num, name=None):
        '''添加联系人
        :Args:
        -phone: 要添加的联系人姓名
        -num: 一共要添加多少个联系人
        -name: 联系人的姓名，没有时，显示电话号码
        '''
        opts = {
             'type':"add",
             'phone':phone,
             'name':name,
             'num': num,
             }
        self._execute(Command.CONTACT, opts)
    
    def clear_contact(self):
        opts = {
             'type':"clear"
            }
        self._execute(Command.CONTACT, opts)
    
    def getFileList(self, targetDir, mode=1, showtype=0):
        '''sd卡目录下，相关文件及大小'''
        opts = {
             'dir':targetDir,  # 目录名
             'mode':mode,  # 查找模式，1 内置sd卡，2 外置sd卡
             'showtype':showtype,  # 返回数据形式， 0 只返回文件名，1 文件名及大小
            }
        return self._execute(Command.File_List, opts)['value']
    
    def startSetting(self, action):
        opts = {
             'action':action
            }
        self._execute(Command.SettingAction, opts)
    
    def setScreenLightMode(self, mode):
        '''mode 1 为自动调节屏幕亮度,0 为手动调节屏幕亮度 '''
#         opts={
#              'type':"set",
#              'mode':mode
#             }
#         self._execute(Command.LIGHT, opts)
        self.adb.setScreenLightMode(mode)
    
    def getScreenLightMode(self):
        '''mode 1 为自动调节屏幕亮度,0 为手动调节屏幕亮度 '''
        opts = {
             'type':"get",
             'mode':'mode'
            }
        return self._execute(Command.LIGHT, opts)['value']
    
    def setRingMode(self, mode):
        '''mode 0 SILENT,1 VIBRATE 2 NORMAL '''
        opts = {
             'type':"set",
             'mode':mode
            }
        self._execute(Command.AUDIO, opts)
    
    def getRingMode(self):
        '''mode 0 SILENT,1 VIBRATE 2 NORMAL '''
        opts = {
             'type':"get",
             'mode':'mode'
            }
        return self._execute(Command.AUDIO, opts)['value']
    
    def openFlashlight(self):
        '''打开闪光灯 '''
        opts = {
             'type':"open",
            }
        self._execute(Command.CAMARA_LIGHT, opts)
    
    def closeFlashlight(self):
        '''关闭闪光灯'''
        opts = {
             'type':"close",
            }
        return self._execute(Command.CAMARA_LIGHT, opts)
    
    def openData(self):
        '''打开闪光灯 '''
        opts = {
             'type':"open",
            }
        self._execute(Command.DATA, opts)
    
    def closeData(self):
        '''关闭闪光灯'''
        opts = {
             'type':"close",
            }
        return self._execute(Command.DATA, opts)
    
    def getDataStatus(self):
        '''获取移动网数据状态'''
        opts = {
             'type':"get",
            }
        return self._execute(Command.DATA, opts)['value']
    
    def _setConfigurator(self, method, value):
        '''设置测试配置参数
        :Args:
        -method: actionAcknowledgmentTimeout(3 * 1000) keyInjectionDelay(0) 
                 scrollAcknowledgmentTimeout(200) waitForIdleTimeout(10 * 1000)
                 waitForSelectorTimeout(10 * 1000)
        -value(ms): 设置操作等待时间，单位是ms
        '''
        opts = {
             'config': method,
             'value': value
            }
        self._execute(Command.CONFIGURATOR, opts)
    
    def getSimInfo(self, category=0):
        '''获取sim卡信息
        :Args:
        -type: 0 主卡
               1 副卡
        '''
        opts = {
             'type': category
            }
        return json.loads(self._execute(Command.SIM_INFO, opts)['value'])
        
    
    def getDeviceInfo(self, mode=0):
        '''获取设备信息
        :Args:
        -mode: 0 基本数据
               1 详细信息，带sd卡，sim卡信息
        '''
        opts = {
             'mode': mode
            }
        return self._execute(Command.DEVICE_INFO, opts)['value']
    
    def getLocalTmpDir(self):
        ''''返回/data/local/tmp临时目录'''
        return self._getDevicePath(0)
    
    def getInternalSdcard(self):
        ''''内置sd卡目录'''
        return self._getDevicePath(1)
    
    def getExtendSdcard(self):
        '''外置sd卡目录'''
        return self._getDevicePath(2)
    
    def _getDevicePath(self, mode=0):
        '''获取设备目录信息
        :Args:
        -mode: 0 临时数据目录
               1 内置sd卡目录
               2 外置sd卡目录
        '''
        opts = {
             'mode': mode
            }
        return self._execute(Command.DEVICE_PATH, opts)['value']
    
    def queryRemoteSiminfo(self, imsi):
        '''获取设备目录信息
        :Args:
        -imsi: 国际移动用户识别码
        '''
        opts = {
             'imsi': imsi
            }
        return self._execute(Command.REMOTE_SIM_INFO, opts)['value']
    
    def getPhoneNum(self, category=0):
        '''获取电话号码信息'''
        siminfo = self.getSimInfo(category).get("imsi")
        return self.queryRemoteSiminfo(siminfo).get("number")
    
    def clearUiCache(self):
        '''部分界面出现，元素无法找到，可能因为缓存区数据未更新导致，目前原因不能，先用此api处理'''
        self._execute(Command.CLEAR_UI_CACHE)
        
    def registerUiWatcher(self, name, text, packageName=None):
        '''注册界面监听器，用于弹窗点掉机制
        app.registerUiWatcher("ll", "允许","com.huawei.systemmanager")
        :Args:
        -name: 全局唯一指定名称，用于添加，删除时的唯一标识
        -text: 出现弹出后，需要点击的文字
        -packageName: 出现弹出后，弹窗所属包名
        '''
        opt = {
              "flag":"add",
              "name":name,
              "text":text,
              "packageName":packageName
              }
        self._execute(Command.UI_WATCHERS, opt)
    
    def removeUiWatcher(self, name):
        '''移除指定ui监听器'''
        opt = {
              "flag":"remove",
              "name":name,
              }
        self._execute(Command.UI_WATCHERS, opt)
    
    def removeAllUiWatcher(self):
        '''移除所有ui监听器'''
        opt = {
              "flag":"removeall",
              }
        self._execute(Command.UI_WATCHERS, opt)
    
    def openQuickPanel(self, duration=None):
        '''上滑打开部分机子显现的快捷面板'''
        self.swipe(0.5, 0.99, 0.5, 0.5, duration)
    
    def get_device_real_size(self):
        '''get device real size'''
        opt = {
               "flag":True
               }
        return self._execute(Command.GET_DEVICE_SIZE, opt)['value']
    
    def pressSearch(self):
        self.press_keycode(84)
        
    def pressEnter(self):
        self.press_keycode(66)
    
    def getContactNum(self):
        '''获取联系人个数'''
        opt = {
               'type':"get"
               }
        return self._execute(Command.CONTACT, opt)['value']
        
    def rmBlackBorder(self, srcpath, tarpath, thres, diff, shrink, directionMore):
        '''图片黑边处理
        app.rmBlackBorder("src.png","result.png" , 50,1000,0,0)
        :Args:
        -srcpath: 源图片路径
        -tarpath: 结果图片路径
        -thres: threshold for cropping: sum([r,g,b] - [0,0,0](black))图像阈值
        -diff: max tolerable difference between black borders on two side
        -shrink: number of pixels to shrink after the blackBorders removed
        -directionMore: 哪个方向上多出不对称的内容，0：正常，1：上，2：下,3：左，4：右
        '''        
        ImageUtil.rmBlackBorder(srcpath, tarpath, thres, diff, shrink, directionMore)
        
    def getWifiHotInfo(self):
        '''获取wifi热点信息'''
        try:
            return ViewRemoteResource.getInstance().getWifiHotInfo()['data']
        except:
            pass
    
    def stop_third_app(self,ignore_filter=["com.tencent.mm"]):
        '''停止第三方app'''
        self.adb.stop_third_app(ignore_filter)
    



# coding=utf-8

import logging, threading
import os, sys, time


# File Level
FILE_LOG_LEVEL = "DEBUG"

# Console Level
CONSOLE_LOG_LEVEL = "INFO"

# logger levels
LEVELS = {"CRITICAL": 50,
          "ERROR": 40,
          "WARNING": 30,
          "INFO": 20,
          "DEBUG": 10,
          "NOTSET": 0,
          }
LOG_FILE_PATH = "./log"
'''
getLogger(name="easyuiautomator", path=None,  saveLogtype=None, unit=None, level="DEBUG")
name :         log tag
path :         log sava path
savelogtype:   logfile sava type :[time, size] split file type
unit       :   by savelogtype time(s) size(Byte)
'''
if(sys.getdefaultencoding() != "utf-8"):
    reload(sys)
    sys.setdefaultencoding('utf-8')

class Logger:
    _instance = None
    _mutex = threading.Lock()
    _debug = False  # False 关闭手机端回复日志
    _Flag = False # 运行日志获取标志位
    _message = "" # 截取日志详细信息
    _FrameWork = "[SYSTEM] " # 系统层标签
    _Fitler_Tag = "[DEBUG]"

    def __init__(self, name=None, logPath=None, saveLogtype=None, unit=None, level="DEBUG"):
        """
        Generate root logger.
        """
        if logPath is None:
            logPath = LOG_FILE_PATH
        logPath = os.path.abspath(logPath)
        self.saveType = ["size", 20*1024*1024]
        if saveLogtype:
            self.saveType = [saveLogtype, unit]
        self._logger = logging.getLogger(name)
        self._logger.setLevel(LEVELS[level])
        self._formatter = logging.Formatter("[%(asctime)s] - %(levelname)s : %(message)s", '%Y-%m-%d %H:%M:%S')
        self.add_console_logger()
        self.add_file_logger(logPath)
        
    @staticmethod
    def getLogger(name="easyuiautomator", path=None,  saveLogtype=None, unit=None, level="DEBUG"):
        if (Logger._instance == None):
            Logger._mutex.acquire()
            if (Logger._instance == None):
                Logger._instance = Logger(name, path, saveLogtype, unit, level)
            Logger._mutex.release()
        return Logger._instance 
           
    @classmethod
    def set_debug(cls, debug=True):
        '''Set the log debug mode '''
        cls._debug = debug
    
    @classmethod
    def set_fitler(cls, tag):
        '''set log filter tag'''
        cls._Fitler_Tag = "[%s]"%tag.upper()

    def add_file_logger(self, logFile=None, level="DEBUG"):
        """
        Generate file writer [RotatingFileHandler].
        """
        if not os.path.exists(logFile):
            os.makedirs(logFile)
        path = os.path.join(logFile,'runntime')
        if self.saveType[0] == "size":
            fh = MyLoggerSizeHandler(path, self.saveType[1])
        elif self.saveType[0] == "time":
            fh = MyLoggerTimeHandler(path, self.saveType[1])
        fh.suffix = "%Y%m%d-%H%M.log"
        fh.setLevel(LEVELS[level])
        fh.setFormatter(self._formatter)
        self._logger.addHandler(fh)

    def add_console_logger(self, level="DEBUG"):
        """
        Generate console writer [StreamHandler].
        """
        ch = logging.StreamHandler()
        ch.setLevel(LEVELS[level])
        ch.setFormatter(self._formatter)
        self._logger.addHandler(ch)
    
    def getMsg(self):
        '''获取运行期间日志内容'''
        return self._message
    
    def setLogOn(self):
        self._Flag = True
        self._message = ""

    def setLogOff(self):
        self._Flag = False
 
    def _format_log(self, logType, msg):
        '''区分上层日志与框架日志'''
        format_msg = str(msg)
        if format_msg.startswith("[FRAMEWORK]"):
            format_msg = format_msg.replace("[FRAMEWORK]","[FRAMEWORK]" + logType )
        elif format_msg.startswith("[UPPER]"):
            format_msg = format_msg.replace("[UPPER]","[UPPER]" + logType )
        else:
            format_msg = self._FrameWork + logType + format_msg
            msg = self._FrameWork + str(msg)
#         if self._Flag == True:
#             if not logType.startswith(self._Fitler_Tag):
#                 self._message += self._format_logPrefix() + format_msg + "\n"
        return msg
        
    def _format_logPrefix(self):
        '''日志前缀''' 
        return str(time.strftime('[%Y.%m.%d-%H.%M.%S] ', time.localtime(time.time())))

    def debug(self, msg):
        msg = self._format_log("[DEBUG] ", msg)
        self._logger.debug(msg)
        
    def info(self, msg):
        msg = self._format_log("[INFO] ", msg)
        self._logger.info(msg)

    def warning(self, msg):
        msg = self._format_log("[WARNING] ", msg)
        self._logger.warning(msg)

    def error(self, msg):
        msg = self._format_log("[ERROR] ", msg)
        self._logger.error(msg)

    def critical(self, msg):
        msg = self._format_log("[CRITICAL] ", msg)
        self._logger.critical(msg)
       
    
class MyLoggerTimeHandler(logging.Handler):
    
    def __init__(self, filePrefix=None, splitTime=2*60*60):
        
        self.filePrefix = filePrefix
        self.last_time = 0
        self.splitTime = splitTime
        self.filePath = ""
        logging.Handler.__init__(self)

    def emit(self, record):
        msg = self.format(record)
        if self.last_time == 0:
            self.last_time = time.time()
        elif time.time() - self.last_time > self.splitTime:
            self.last_time = time.time()
        file_name = time.strftime('%Y%m%d-%H%M%S', time.localtime(self.last_time))
        self.filePath = self.filePrefix + file_name + ".log"
        with open(self.filePath, 'a') as f:
            f.write(msg+"\n")
            f.flush()
            
class MyLoggerSizeHandler(logging.Handler):
    
    def __init__(self, filePrefix=None, splitSize=20*1024*1024):
        
        self.filePrefix = filePrefix
        self.splitSize = splitSize
        self.filePath = None
        logging.Handler.__init__(self)

    def emit(self, record):
        msg = self.format(record)
        file_name = time.strftime('%Y%m%d-%H%M%S', time.localtime(time.time()))
        if self.filePath:
            size = os.path.getsize(self.filePath)
            if size > self.splitSize:
                self.filePath = self.filePrefix + file_name + ".log"     
        else:
            self.filePath = self.filePrefix + file_name + ".log"
        with open(self.filePath, 'a') as f:
            f.write(msg+"\n")
            f.flush()
                



#coding=utf-8
'''
Created on 2017年7月12日

@author: Administrator
'''
import threading, json
import requests

root_path = "http://172.16.12.46/raptor_back/web/testresources/"
sim = root_path + "sim"
wifi = root_path + "wifi"

class ViewRemoteResource(object):
    '''查看服务器远程资源，用于自动化辅助能力
    '''
    headers={"Content-Type": "application/json"}
    __instance = None
    __mutex = threading.Lock()
    
    def __init__(self):
        pass
    
    @staticmethod
    def getInstance():
        """Return a single instance of ViewRemoteResource object """
        if (ViewRemoteResource.__instance == None):
            ViewRemoteResource.__mutex.acquire()
            if (ViewRemoteResource.__instance == None):
                ViewRemoteResource.__instance = ViewRemoteResource()
            ViewRemoteResource.__mutex.release()
        return ViewRemoteResource.__instance
    
    def _query_data(self, url, data):
        '''查询服务器数据'''
        r = requests.post(url, data=json.dumps(data), headers=self.headers, timeout=10)
        return json.loads(r.content)
    
    def getSimInfo(self, iccid):
        data = {'imsi': iccid}
        return self._query_data(sim, data)
    
    def getWifiHotInfo(self):
        '''获取wifi热点信息'''
        data = {'ip': "tmp"}
        return self._query_data(wifi, data)
    
if __name__ == '__main__':
    print ViewRemoteResource.getInstance().getWifiHotInfo()['data']
        


# coding=utf-8

"""
Created on 2015年11月5日

@author: thomas.ning
"""


import copy
from easyuiautomator.driver.common.mobilecommand import MobileCommand as Command


class MultiAction(object):
    
    def __init__(self, driver, element=None):
        self._driver = driver
        self._element = element
        self._touch_actions = []

    def add(self, *touch_actions):
        """Add TouchAction objects to the MultiAction, to be performed later.

        :Args:
         - touch_actions - one or more TouchAction objects describing a chain of actions to be performed by one finger
        :Usage:
            a1 = TouchAction(driver)
            a1.press(el1).move_to(el2).release()
            a2 = TouchAction(driver)
            a2.press(el2).move_to(el1).release()

            MultiAction(driver).add(a1, a2)
        """
        for touch_action in touch_actions:
            if self._touch_actions is None:
                self._touch_actions = []
            # deep copy, so that once they are in here, the user can't muck about
            self._touch_actions.append(copy.copy(touch_action))
        return self

    def perform(self):
        """Perform the actions stored in the object.
        :Usage:
            a1 = TouchAction(driver)
            a1.press(el1).move_to(el2).release()
            a2 = TouchAction(driver)
            a2.press(el2).move_to(el1).release()
            MultiAction(driver).add(a1, a2).perform()
        """
        self._driver._execute(Command.MULTI_ACTION, self.json_wire_gestures)
        # clean up and be ready for the next batch
        self._touch_actions = []
        return self

    @property
    def json_wire_gestures(self):
        actions = []
        for action in self._touch_actions:
            actions.append(action.json_wire_gestures)
        if self._element is not None:
            return {'actions': actions, 'elementId': self._element.id}
        else:
            return {'actions': actions} 


# coding=utf-8

"""
Created on 2015年11月6日

@author: thomas.ning
"""

import json,tempfile,zipfile,re,time
import os,base64,sys
try:
    from PIL import Image, ImageChops
except:
    pass 
from easyuiautomator.common.exceptions import NoSuchElementException
from easyuiautomator.common.log import Logger
from easyuiautomator.common.find_img import find_img_position
from easyuiautomator.common.blackCheck import blackBorderCheck  
 
LOGGER = Logger.getLogger()


def format_json(json_struct):
    return json.dumps(json_struct, indent=4)


def dump_json(json_struct, ensure_ascii=True):
    return json.dumps(json_struct, ensure_ascii=ensure_ascii)


def load_json(s):
    return json.loads(s)


def handle_find_element_exception(e):
    if ("No element found" in e["value"] or
                "Unable to locate" in e["value"]):
        raise NoSuchElementException("Unable to locate element:")
    else:
        raise e


def return_value_if_exists(resp):
    if resp and "value" in resp:
        return resp["value"]


def get_root_parent(elem):
    parent = elem.parent
    while True:
        try:
            parent.parent
            parent = parent.parent
        except AttributeError:
            return parent


def unzip_to_temp_dir(zip_file_name):
    """Unzip zipfile to a temporary directory.

    The directory of the unzipped files is returned if success,
    otherwise None is returned. """
    if not zip_file_name or not os.path.exists(zip_file_name):
        return None

    zf = zipfile.ZipFile(zip_file_name)

    if zf.testzip() is not None:
        return None

    # Unzip the files into a temporary directory
    LOGGER.info("Extracting zipped file: %s" % zip_file_name)
    tempdir = tempfile.mkdtemp()

    try:
        # Create directories that don't exist
        for zip_name in zf.namelist():
            # We have no knowledge on the os where the zipped file was
            # created, so we restrict to zip files with paths without
            # charactor "\" and "/".
            name = (zip_name.replace("\\", os.path.sep).
                    replace("/", os.path.sep))
            dest = os.path.join(tempdir, name)
            if (name.endswith(os.path.sep) and not os.path.exists(dest)):
                os.mkdir(dest)
                LOGGER.debug("Directory %s created." % dest)

        # Copy files
        for zip_name in zf.namelist():
            # We have no knowledge on the os where the zipped file was
            # created, so we restrict to zip files with paths without
            # charactor "\" and "/".
            name = (zip_name.replace("\\", os.path.sep).
                    replace("/", os.path.sep))
            dest = os.path.join(tempdir, name)
            if not (name.endswith(os.path.sep)):
                LOGGER.debug("Copying file %s......" % dest)
                outfile = open(dest, 'wb')
                outfile.write(zf.read(zip_name))
                outfile.close()
                LOGGER.debug("File %s copied." % dest)

        LOGGER.info("Unzipped file can be found at %s" % tempdir)
        return tempdir

    except IOError as err:
        LOGGER.error("Error in extracting webdriver.xpi: %s" % err)
        return None


def formatReturnStr(status, value):
    return {'status': status, 'value': value}

def formatExceptionStr(device, msg):
    return dump_json({'device': device, 'message': msg}, False)

def is_base64(strword):
    try:
        if len(strword) < 200:
            if os.path.exists(strword):
                return False
        else:
            base64.b64decode(strword)
            return True
    except:
        pass
    return False

def waitForCondition(method, returnValue, timeout=5, intervalMs=0.5, args=None):
    '''等待条件超时函数，根据函数返回值条件，确认是否跳出
    ：    :Args:
        methon: 函数名
        args: 必须是元组参数传递形式，如（'tt',dd)
        returnValue:对比值
    '''
    end_time = time.time() + timeout
    while True:
        try:
            if args is None:
                tmp = method()
            else:
                tmp = method(*args)
            if tmp == returnValue:
                return True
        except:
            print sys.exc_info()
        if time.time() - end_time>0:
            return None
        time.sleep(intervalMs)    

class ImageUtil:
    '''
    image deal class
    '''
    @staticmethod
    def find_image_positon(query, origin, algorithm='sift', radio=0.75):
        temp = origin
        if is_base64(origin):
            temp=tempfile.mktemp()
            with open(temp,"wb") as f:
                f.write(origin)
        if not os.path.exists(query):
            raise IOError,('No such file or directory:%s'%query)
        position = find_img_position(query, temp, algorithm, radio)
        if os.path.exists(temp):os.remove(temp)
        return position

    @staticmethod
    def compare_stream(strStream, target_file):
        '''
        file stream compare
        :Args:
         - strStream: strStrem by driver.get_screenshot_as_png.
         - target_file: need compared target file.
        '''
        temp=tempfile.mktemp()
        with open(temp,"wb") as f:
            f.write(strStream)
        simily = ImageUtil.compare(target_file, temp)
        if os.path.exists(temp):os.remove(temp)
        return simily
    
    @staticmethod
    def compare(f1, f2):
        """
        Calculate the similarity  between f1 and f2
        return similarity  0-100
        """
        img1 = Image.open(f1)
        img2 = Image.open(f2,'r')
        # if image size is not equal, return 1
        if img1.size[0] != img2.size[0] or img1.size[1] != img2.size[1]:
            return 0
        size = (256, 256)
        img1 = img1.resize(size).convert('RGB')
        img2 = img2.resize(size).convert('RGB')
        # # get the difference between the two images
        h = ImageChops.difference(img1, img2)
        size = float(img1.size[0] * img1.size[1])
        diff = 0
        for p in list(h.getdata()):
            if p != (0, 0, 0):
                diff += 1
        return round((1 - (diff / size)) * 100, 2)

    @staticmethod
    def crop(startx, starty, endx, endy, scrfile, destfile):
        """
        cut img by the given coordinates and picture, then make target file
        """
        box = (startx, starty, endx, endy)
        img = Image.open(scrfile)
        cut_img = img.crop(box)
        if cut_img:
            cut_img.save(destfile)
            return True
        else:
            return False
        
    @staticmethod
    def rmBlackBorder(
                srcpath,  # input image
                tarpath,
                thres,  # threshold for cropping: sum([r,g,b] - [0,0,0](black))
                diff,  # max tolerable difference between black borders on two side
                shrink,  # number of pixels to shrink after the blackBorders removed
                directionMore  # 哪个方向上多出不对称的内容，0：正常，1：上，2：下,3：左，4：右
                ):
        blackBorderCheck(srcpath,tarpath,thres,diff,shrink,directionMore)
        

######################
class Verify:
    """
    Class verify is specially used for element attribute validation.
    Including Text、Image、Content_Desc and etc.
    """
    @staticmethod
    def verifyText(srcText, desText):
        """
        Verify element's Text of attribute whether is equal to the text you set.
        :Args: text
        :Return: True or False
        """
        if srcText == desText:
            return True
        return False

    @staticmethod
    def verifyNotText(srcText, desText):
        """
        This is the opposite of verifyText.
        :Args: text
        :Return: True or False
        """
        return not Verify.verifyText(srcText, desText)

    @staticmethod
    def verifyImage(srcImage, dstImage, similarity=100):
        """
        Verify element's Image whether is equal to the image you set.
        :Args: image
        :Return: similarity  0-100
        """
        return ImageUtil.compare(srcImage,dstImage) >= similarity

    @staticmethod
    def verifyTextRe(searchText, checkText):
        """
        Verify element's Text of attribute whether is about equal to the text you set.
        :Args: text
        :Return: True or False
        """
        if re.search(checkText,searchText):
            return True
        return False

    @staticmethod
    def verifyNotTextRe(searchText, checkText):
        """
        This is the opposite of verifyTextRe
        :Args: text
        :Return: True or False
        """
        return not Verify.verifyTextRe(searchText, checkText)
if __name__ == '__main__':

    path = r"C:\Program Files\AutoRecorder\workspace\test_tool\tools\test.png"
    path2 = r"C:\Program Files\AutoRecorder\workspace\test_tool\tools\test2.png"
    print ImageUtil.rmBlackBorder(path, path2, 50,1000,0,0)


# coding=utf-8
'''
Created on 2015年11月5日

@author: thomas.ning
'''

class MobileCommand(object):

    CONTEXTS = 'getContexts',
    GET_CURRENT_CONTEXT = 'getCurrentContext',
    SWITCH_TO_CONTEXT = 'switchToContext'
    GET_NETWORK_CONNECTION = 'getNetworkConnection'
    SET_NETWORK_CONNECTION = 'setNetworkConnection'
    GET_AVAILABLE_IME_ENGINES = 'getAvailableIMEEngines'
    IS_IME_ACTIVE = 'isIMEActive'
    ACTIVATE_IME_ENGINE = 'activateIMEEngine'
    DEACTIVATE_IME_ENGINE = 'deactivateIMEEngine'
    GET_ACTIVE_IME_ENGINE = 'getActiveEngine'
    TOGGLE_LOCATION_SERVICES = 'toggleLocationServices'
    LOCATION_IN_VIEW = 'locationInView'
    SET_IMMEDIATE_VALUE = 'setImmediateValue'
    REPLACE_KEYS = 'replaceKeys'
    GET_SETTINGS = 'getSettings'
    UPDATE_SETTINGS = 'updateSettings'


#################################################################################################

    '''
    drive uiautomator action
    '''
    FIND_ELEMENT = "findElement"
    FIND_ELEMENTS = "findElements"
    GET_PARENT = "getParent"
    IS_EXIST = "isExist"
    PRESS_KEYCODE = 'pressKeyCode'
    PRESS_BACK = "pressBack"
    LONG_PRESS_KEYCODE = 'longPressKeyCode'
    TOUCH_ACTION = 'touchAction'
    MULTI_ACTION = 'multiAction'
    WAKE = "wake"
    ZOOM = "zoom"
    PINCH = "pinch"
    DRAG = "drag"
    FLICK = "flick"
    SWIPE = "swipe"
    SWIPE_ORIENTAION = "swipeOrientation"
    SCROLL_TO = "scrollTo"
    SCROLL_ORIENTAION = "scrollOrientation"
    GET_DATA_DIR = "getDataDir"
    SCREENSHOT = "screenshot"
    COMPRESSD_LAYOUT_HIERARCHY = "compressedLayoutHierarchy"
    GET_DEVICE_SIZE = "getDeviceSize"
    OPEN_NOTIFICATION = "openNotification"
    OPEN_RECENT_APPS = "openRecentApp"
    OPEN_QUICK_SETTING = "openQuickSetting"
    ROTATION = "rotation"
    START_APP = "startApp"
    UPDATE_STRINGS = "updateStrings"
    GET_PAGE_SOURCE = "getPageSource"
    LOCK = 'lock'
    UNLOCK = "unlock"
    WAIT_FOR_IDLE = "waitForIdle"
    ORIENTAION = "orientation"
    SHUTDOWN = "shutdown"
    PING = "ping"
    CLEAR_CACHE="clearCache"
    CLICK = "click"
    LONG_CLICK = "touchLongClick"
    
    '''
    element action
    '''
    FINE_CHILD_ELEMNT = "findChildElement"
    FINE_CHILD_ELEMNTS = "findChileElements"
    CLEAR_ELEMENT = "clearElement"

    GET_ATTRIUBTE = "getAttribute"
    GET_LOCATION = "getLocation"
    GET_SIZE = "getSize"
    GET_TEXT = "getText"
    SET_TEXT = "setText"
    GET_NAME = "getName"
    GET_BOUNDS = "getBounds"
    SET_LOCATION = 'setLocation'
    
    '''
    common
    '''
    TAKE_SCREEN_SHOT = "takeScreenshot"
    SET_TIMEOUT = "setTimout"
    SET_THINKTIME = "setThinktime"
    BATTERY_CTRL = "batteryCtrl"
    MONITOR = "monitor" # 监听电量 adbd
    
    '''
    driver adb
    '''
    START_ACTIVITY = 'startActivity'
    GET_CURRENT_APP = 'getCurrentAPP'
    HIDE_KEYBOARD = 'hideKeyboard'
    PULL_FILE = 'pullFile'
    PULL_FOLDER = 'pullFolder'
    PUSH_FILE = 'pushFile'
    SHAKE = 'shake'
    RESET = 'reset'
    BACKGROUND = 'background'
    IS_APP_INSTALLED = 'isAppInstalled'
    INSTALL_APP = 'installApp'
    INSTALL_REMOTE_APP = 'installRemoteApp'
    REMOVE_APP = 'removeApp'
    LAUNCH_APP = 'launchApp'
    CLOSE_APP = 'closeApp'
    DO_ADB = 'doAdb'
    IS_ONLINE = 'is_online'
    IS_LOCKED = 'is_locked'
    TOAST = "toast"
    WIFI = "wifi"
    BLUETOOTH = "bluetooth"
    CONTACT = "contact"
    CALL_LOG = "calllog"
    GPS = "gps"
    SettingAction = "settingAction"
    File_List = "fileList"
    LIGHT = "light"
    AUDIO = "audio"
    CAMARA_LIGHT = "camaraLight"
    DATA = "data"
    CONFIGURATOR = "configurator"
    DEVICE_INFO = "deviceinfo"
    DEVICE_PATH = "devicepath"
    SIM_INFO = "sim_info"
    REMOTE_SIM_INFO = "remote_sim_info"
    CLEAR_UI_CACHE = "clearUiCache"
    UI_WATCHERS = "ui_watcher"
    SEND_KEYS = "sendKeys"
    



# coding=utf-8

"""
Created on 2015年11月6日

@author: thomas.ning
"""
import time,math,utils,copy,os,re
from adb import ADB
from uiautomator import Uiautomator
from easyuiautomator.common.exceptions import DriverException
from easyuiautomator.common.exceptions import TimeoutException
from easyuiautomator.common.exceptions import AdbException
from easyuiautomator.common.exceptions import UiautomatorException
from easyuiautomator.driver.common.mobilecommand import MobileCommand as Command
from easyuiautomator.common.log import Logger
from easyuiautomator.common.view_remote_resource import ViewRemoteResource
from easyuiautomator.common.exceptions import RemoteServiceException

ip_match = "^\d+\.\d+\.\d+\.\d+$"

class CommandExecutor(object):

    """命令执行器，用来执行adb命令及远程uiautomator命令字"""
    _thinkTime = 0.5  #命令执行前等待时间，已启用
    _timeOut = None # 命令执行超时时长，未启用

    def __init__(self, device_id=None, port=None, restart=True):
        self.logger = Logger.getLogger()
#         if re.match(ip_match, device_id) == False:
        self._adb = ADB(device_id, restart)
        self.sdk_version = self._adb.getApiLevel()
        self.logger.debug("sdk version %s on %s"%(self.sdk_version, self._adb.serial))
        self._uiautomator = Uiautomator(adb=self._adb, sdk_version=self.sdk_version, local_port=port)
        self._uiautomator.start()  # 启动uiautomator远程服务
        # 电量监控相关参数
        self.powerMonitor = PowerMonitor(self._adb)
        self._controller = Controller(self._adb, self._uiautomator, self._timeOut)
        # commandExecutor support all command word
        self._commands = {
            Command.FIND_ELEMENT: self._controller.findElement,
            Command.FIND_ELEMENTS: self._controller.findElements,
            Command.FINE_CHILD_ELEMNT: self._controller.findChildElement,
            Command.FINE_CHILD_ELEMNTS: self._controller.findChildElements,
            Command.ZOOM: self._controller.zoom,
            Command.PINCH: self._controller.pinch,
            Command.SET_TIMEOUT: self._controller.setTimeout,
            Command.CLICK: self._controller.click,
            Command.LONG_CLICK: self._controller.longclick,
            Command.WIFI: self._controller.wifi,
            Command.CLEAR_ELEMENT: self._controller.clear,
            Command.IS_EXIST: self._controller.isExist,
            Command.GET_TEXT: self._controller.getText,
            Command.SET_TEXT: self._controller.setText,
            Command.GET_SIZE: self._controller.getSize,
            Command.GET_LOCATION: self._controller.getLocation,
            Command.GET_ATTRIUBTE: self._controller.getAttribute,
            Command.LOCK: self._controller.lock,
            Command.WAKE: self._controller.wake,
            Command.GET_DEVICE_SIZE: self._controller.getDeviceSize,
            Command.COMPRESSD_LAYOUT_HIERARCHY: self._controller.compressedLayoutHierarchy,
            Command.OPEN_NOTIFICATION: self._controller.openNotification,
            Command.GET_DATA_DIR: self._controller.getDataDir,
            Command.GET_PAGE_SOURCE: self._controller.source,
            Command.GET_PARENT: self._controller.getParent,
            Command.START_APP: self._controller.startApp,
#             Command.SCREENSHOT: self._controller.takeScreenshot,
            Command.TOUCH_ACTION: self._controller.touchAction,
            Command.MULTI_ACTION: self._controller.multiAction,
            Command.PRESS_KEYCODE: self._controller.pressKeyCode,
            Command.LONG_PRESS_KEYCODE: self._controller.longPressKeyCode,
            Command.PRESS_BACK: self._controller.goBack,
            Command.SCROLL_TO: self._controller.scrollTo,
            Command.SCROLL_ORIENTAION: self._controller.scrollOrientation,
            Command.SWIPE : self._controller.swipe,
            Command.SWIPE_ORIENTAION : self._controller.swipeOrientation,
            Command.ORIENTAION : self._controller.orientation,
            Command.DRAG : self._controller.drag,
            Command.SCROLL_ORIENTAION:self._controller.swipeDirection,
            Command.MONITOR:self._controller.monitor,
            Command.CLEAR_CACHE:self._controller.clearCache,
            Command.OPEN_QUICK_SETTING:self._controller.openQuickSetting,
            Command.OPEN_RECENT_APPS:self._controller.openRecentApp,
            Command.ROTATION:self._controller.roatation,
            Command.TOAST: self._controller.toast,
            Command.WAIT_FOR_IDLE: self._controller.waitForIdle,
            Command.CALL_LOG: self._controller.call_log,
            Command.CONTACT: self._controller.contact,
            Command.BLUETOOTH: self._controller.bluetooth,
            Command.GPS: self._controller.gps,
            Command.SettingAction: self._controller.settingAction,
            Command.File_List: self._controller.fileList,
            Command.LIGHT: self._controller.light,
            Command.AUDIO: self._controller.audio,
            Command.CAMARA_LIGHT: self._controller.flashLight,
            Command.DATA: self._controller.data,
            Command.CONFIGURATOR: self._controller.configurator,
            Command.DEVICE_INFO: self._controller.device_info,
            Command.DEVICE_PATH: self._controller.device_path,
            Command.SIM_INFO: self._controller.sim_info,
            Command.CLEAR_UI_CACHE: self._controller.clearUiCache,
            Command.UI_WATCHERS: self._controller.uiWatcher,
            Command.SEND_KEYS: self._controller.sendKeys,
            
            #############remote######################
            Command.REMOTE_SIM_INFO: self._controller.remote_sim_info,

            ####### ADB #######
            Command.START_ACTIVITY: self._controller.startActivity,
#             Command.SCREENSHOT: self._controller.takeScreenshot_adb,
             Command.SCREENSHOT: self._controller.takeScreenshot,
            Command.GET_CURRENT_APP: self._controller.getCurrentApp,
            Command.HIDE_KEYBOARD: self._controller.hideKeyboard,
            Command.PULL_FILE: self._controller.pullFile,
            Command.PULL_FOLDER: self._controller.pullFolder,
            Command.PUSH_FILE: self._controller.pushFile,
            Command.RESET: self._controller.reset,
            Command.SHAKE: self._controller.shake,
            Command.BACKGROUND: self._controller.background,
            Command.IS_APP_INSTALLED: self._controller.isAppInstalled,
            Command.INSTALL_APP: self._controller.installApp,
            Command.INSTALL_REMOTE_APP: self._controller.installRemoteApp,
            Command.REMOVE_APP: self._controller.removeApp,
            Command.LAUNCH_APP: self._controller.lanuchApp,
            Command.CLOSE_APP: self._controller.closeApp,
            Command.DO_ADB: self._controller.doAdb,
            Command.IS_ONLINE: self._controller.isOnline,
            Command.IS_LOCKED: self._controller.isScreenLocked
        }

    def execute(self, driver_command, params):
        """
        Sends a command to be executed by a command.CommandExecutor.
        :Args:
         - driver_command: The name of the command to execute as a string.
         - params: A dictionary of named parameters to send with the command.
        :Returns:
          The command's JSON response loaded into a dictionary object.
        """
        command_info = self._commands.get(driver_command)
        assert command_info is not None, 'Unrecognised command %s' % driver_command
        if params is not None:
            if params.get("thinkTime"):
                time.sleep(params.get("thinkTime"))
            elif self._thinkTime:
                time.sleep(self._thinkTime)
        elif self._thinkTime:
            time.sleep(self._thinkTime)
        self.logger.info("[%s exec %s params:%s]" % (self._adb.serial, driver_command, utils.dump_json(params, ensure_ascii=False)))
        if driver_command == Command.MONITOR:
            power_status = params["power_status"]
            height = params["high"] if params.get("high") else 40
            low = params["low"] if params.get("low") else 10
            self.powerMonitor.setMonitorParams(power_status, height, low)
            return
        self.powerMonitor.waitPowerStatus() # 查看手机电量情况
        reponse = command_info(params)
        return reponse

    def stop(self):
        self._adb.restart = False
        self._uiautomator.stop()

class PowerMonitor:
    
    def __init__(self, adb):
        self.logger = Logger.getLogger()
        self.adb = adb
        self.charging = False
        self.power_monitor = False
        self.power_high = 40
        self.power_low = 10
        self.current_power_level = 0
        self.lastCheckTime = 0
        
    def setMonitorParams(self,power_monitor, power_high, power_low):
        self.power_monitor = power_monitor
        self.power_high = power_high
        self.power_low = power_low
        
    def waitPowerStatus(self):
        if time.time() - self.lastCheckTime < 5*60: # 电量正常时，每隔5分钟读取电量一次
            return
        while self.power_monitor:
            if self.checkPowerLevelThresold():
                time.sleep(5*60)
            else:
                break
        self.lastCheckTime = time.time()
        
    def checkPowerLevelThresold(self):
        '''true: 充电  false: 未充电'''
        self._setPowerLevel()
        if self.current_power_level > self.power_high:
            self.charging = False
            return False
        if self.current_power_level < self.power_low:
            self.charging = True
            self.logger.warning("%s 正在充电中，当前电量%s"%(self.adb.serial,self.current_power_level))
            return True
        if self.current_power_level < self.power_high \
        and self.current_power_level > self.power_low:
            if self.charging:
                self.logger.warning("%s 正在充电中，当前电量%s"%(self.adb.serial,self.current_power_level))
                return True
            return False

    def _setPowerLevel(self):
        current_power_level = self.adb.get_battery()
        if current_power_level is None:
            self.current_power_level = 100
            self.logger.warning("电量无法正常获取,电量监控功能失效")
            return
        if current_power_level > 0:
            try:
                self.current_power_level = int(current_power_level)
            except:
                pass


class Controller(object):
    """    
    Controller provide all basic action routing
    """
    
    def __init__(self, adb, uiautomator, timeout):
        self.timeout = timeout  # 查找元素超时时长
        self.steps = 20  # 默认滑动步长
        self._adb = adb
        self._uiautomator = uiautomator

    def compressedLayoutHierarchy(self, params):
        result = self._uiautomator.sendActon("compressedLayoutHierarchy", params)
        return self._deal_result(result)

    def getDataDir(self, params):
        result = self._uiautomator.sendActon("getDataDir", params)
        return self._deal_result(result)

    def source(self, params):
        result = self._uiautomator.sendActon("source", params)
        return self._deal_result(result)

    def getDeviceSize(self, params):
        result = self._uiautomator.sendActon("getDeviceSize", params)
        return self._deal_result(result)

    def openNotification(self, params):
        result = self._uiautomator.sendActon("openNotification", params)
        return self._deal_result(result)

    def takeScreenshot(self, params):
        opts = {
              "scale": params["scale"] if params.get("scale") else 1.0,
              "quality": params["quality"] if params.get("quality") else 50
              }
        if params.get("id"):
            opts["elementId"] = params["id"]
            result = self._uiautomator.sendActon("element:takeScreenshot", opts)
        else:
            result = self._uiautomator.sendActon("takeScreenshot", opts)
        return self._deal_result(result)

    def findElements(self, params):
        params["multiple"] = True
        result = self._find(params)
        return self._deal_result(result)

    def findElement(self, params):
        params['multiple'] = False
        result = self._find(params)
        return self._deal_result(result)

    def findChildElement(self, params):
        params['multiple'] = False
        result = self._find(params)
        return self._deal_result(result)

    def findChildElements(self, params):
        params['multiple'] = True
        result = self._find(params)
        return self._deal_result(result)

    def _find(self, params):
        opts = {
            'strategy': params["using"],
            'selector': params["value"],
            'multiple': params['multiple'],
            'context': params["id"] if params.get("id") else ""
        }
        if opts['strategy'] == "xpath" \
                and opts['context'] != "":
            raise DriverException("Cannot use xpath locator strategy from an element."
                                  "It can only be used from the root element")
        if params['timeOut'] is not None or self.timeout is not None:
            return self._waitForCondition(self._uiautomator.sendActon, ("find", opts), params['timeOut'])
        return self._uiautomator.sendActon("find", opts)
        

    def _waitForCondition(self, method, args, timeout=None, intervalMs=0.5):
        if timeout is None:
            timeout = self.timeout
        end_time = time.time() + timeout
        while True:
            response = method(args[0], args[1])
            if response != None and response['status'] == 0 and response['value'] is not None:
                return response
            elif time.time() - end_time > 0:
                raise TimeoutException(utils.formatExceptionStr(self._adb.serial, response['value'] if response else None))
            time.sleep(intervalMs)

    def setTimeout(self, params):
        ms = params['ms']
        self.timeout = ms

    def click(self, params):
        opts = {}
        opts['x'] = params["x"] if params.get("x") else 1
        opts["y"] = params["y"] if params.get("y") else 1
        if params.get("id"):
            opts["elementId"] = params["id"]
            result = self._uiautomator.sendActon("element:click", opts)
        else:
            result = self._uiautomator.sendActon("click", opts)
        return self._deal_result(result)

    def longclick(self, params):
        opts = {}
        opts['x'] = params["x"] if params.get("x") else 1
        opts["y"] = params["y"] if params.get("y") else 1
        if params.get("duration") is not None:
            opts["duration"] = params["duration"]
        if params.get("id"):
            opts["elementId"] = params["id"]
            result = self._uiautomator.sendActon("element:touchLongClick", opts)
        else:
            result = self._uiautomator.sendActon("touchLongClick", opts)
        return self._deal_result(result)

    def touchDown(self, params):
        opts = {}
        if params.get('id'): opts["elementId"] = params["id"]
        opts['x'] = params["x"] if params.get("x") else 1
        opts["y"] = params["y"] if params.get("y") else 1
        result = self._uiautomator.sendActon("element:touchDown", opts)
        return self._deal_result(result)

    def touchMove(self, params):
        opts = {}
        if params.get("id"): opts["elementId"] = params["id"]
        opts['x'] = params["x"] if params.get("x") else 1
        opts["y"] = params["y"] if params.get("y") else 1
        result = self._uiautomator.sendActon("element:touchMove", opts)
        return self._deal_result(result)

    def touchUp(self, params):
        opts = {}
        if params.get("id"): opts["elementId"] = params["id"]
        opts['x'] = params["x"] if params.get("x") else 1
        opts["y"] = params["y"] if params.get("y") else 1
        result = self._uiautomator.sendActon("element:touchUp", opts)
        return self._deal_result(result)

    def touchAction(self, params):
        command = {
            'tap': self.touchDown,
            'press': self.touchDown,
            'release': self.touchUp,
            'moveTo': self.touchMove,
            'wait': self._wait,
            'longPress': self.longclick,
        }
        actions = params.get('actions')
        for tmpaction in actions:
            action = tmpaction.get('action', None)
            opts = tmpaction.get('options')
            command_info = command[action]
            command_info(opts)
        return utils.formatReturnStr(0, "")


    def multiAction(self, params):
        actions = params.get('actions')
        touch_list = []
        for action in actions:
            touch_list.append(self._gesture(action))
        actions = {'actions': touch_list}
        result = self._uiautomator.sendActon("performMultiPointerGesture", actions)
        return self._deal_result(result)

    def _gesture(self, actions):
        '''处理多点触控行为，返回uiautomator识别的指令'''
        opts = []
        tmp = {}
        touch_action = ['press', 'moveTo', 'tap', 'longPress']
        offset = 0.0
        for action in actions:
            if action.get('action') == 'release':
                offset = 0.0
                continue
            if action.get('action') in touch_action:
                offset += 0.005
                options = action.get("options")
                tmp = copy.copy(options)
                touch = {
                    'action': action.get('action'),
                    'touch': options,
                    'time': round(offset, 3)
                }
                opts.append(touch)
            elif action.get('action') == 'wait':
                options = action.get("options")
                tmp['ms'] = options['ms']
                offset += options['ms'] / 1000.0
                touch = {
                    'action': 'wait',
                    'touch': tmp,
                    'time': round(offset, 3),
                }
                opts.append(touch)
        return opts
    
    def wifi(self,params):
        result = self._uiautomator.sendActon("wifi", params)
        return self._deal_result(result)
    
    def settingAction(self,params):
        result = self._uiautomator.sendActon("settingAction", params)
        return self._deal_result(result)
    
    def call_log(self,params):
        result = self._uiautomator.sendActon("calllog", params)
        return self._deal_result(result)
    
    def gps(self,params):
        result = self._uiautomator.sendActon("gps", params)
        return self._deal_result(result)
    
    def bluetooth(self,params):
        result = self._uiautomator.sendActon("bluetooth", params)
        return self._deal_result(result)
    
    def contact(self,params):
        result = self._uiautomator.sendActon("contact", params)
        return self._deal_result(result)
    
    def light(self,params):
        result = self._uiautomator.sendActon("light", params)
        return self._deal_result(result)
    
    def audio(self,params):
        result = self._uiautomator.sendActon("audio", params)
        return self._deal_result(result)
    
    def flashLight(self,params):
        result = self._uiautomator.sendActon("camaraLight", params)
        return self._deal_result(result)
    
    def data(self,params):
        result = self._uiautomator.sendActon("data", params)
        return self._deal_result(result)
    
    def configurator(self,params):
        result = self._uiautomator.sendActon("configurator", params)
        return self._deal_result(result)
    def device_info(self,params):
        result = self._uiautomator.sendActon("deviceinfo", params)
        return self._deal_result(result)
    
    def device_path(self,params):
        result = self._uiautomator.sendActon("devicepath", params)
        return self._deal_result(result)
    
    def sim_info(self,params):
        result = self._uiautomator.sendActon("sim_info", params)
        return self._deal_result(result)
    
    def _wait(self, secs):
        if isinstance(secs, int):
            ms = secs
        elif isinstance(secs, dict):
            ms = secs.get('ms', None)
        time.sleep(ms)
        
    def isExist(self, params):
        opts = {
            "elementId": params["id"],
        }
        result = self._uiautomator.sendActon("element:isExist", opts)
        return self._deal_result(result)

    def zoom(self, params):
        opts = {
            "direction": 'out',
            "elementId": params["id"],
            "percent": params["percent"],
            "steps": params["steps"]
        }
        result = self._uiautomator.sendActon("element:pinch", opts)
        return self._deal_result(result)

    def pinch(self, params):
        opts = {
            "direction": 'in',
            "elementId": params["id"],
            "percent": params["percent"],
            "steps": params["steps"]
        }
        result = self._uiautomator.sendActon("element:pinch", opts)
        return self._deal_result(result)

    def longPressKeyCode(self, params):
        result = self._uiautomator.sendActon("longPressKeyCode", params)
        return self._deal_result(result)

    def pressKeyCode(self, params):
        result = self._uiautomator.sendActon("pressKeyCode", params)
        return self._deal_result(result)
#         self.pressKeyCode_adb(params['keycode'])
    
    def goBack(self, params):
        result = self._uiautomator.sendActon("pressBack", params)
        return self._deal_result(result)

    def clear(self, params):
        opts = {
            'elementId': params['id']
        }
        result = self._uiautomator.sendActon("element:clear", opts)
        return self._deal_result(result)
    
    def startApp(self,params):
        opts = {
            'packageName': params['package'],
            'mode':params['mode']
        }
        result = self._uiautomator.sendActon("startApp", opts)
        return self._deal_result(result)

    def drag(self, params):
        opts = {
            "elementId": params["id"] if params.get("id") else None,
            "destElId":params["destElId"] if params.get("destElId") else None,
            "startX": params["startX"] if params.get("startX") else 1,
            "startY": params["startY"] if params.get("startY") else 1,
            "endX": params["endX"] if params.get("endX") else 1,
            "endY": params["endY"] if params.get("endY") else 1,
            "steps":(int)(math.ceil(params["duration"] * 20)) if params.get("duration") else self.steps
        }
        if params.get('id'):
            result = self._uiautomator.sendActon("element:drag", opts)
        else:
            result = self._uiautomator.sendActon("drag", opts)
        return self._deal_result(result)

    def swipe(self, params):
        opts = {
            "startX": params["startX"],
            "startY": params["startY"],
            "endX": params["endX"],
            "endY": params["endY"],
            "steps": (int)(math.ceil(params["duration"] * 20)) if params.get("duration") else self.steps
        }
        if params.get('id'):
            result = self._uiautomator.sendActon("element:swipe", opts)
        else:
            result = self._uiautomator.sendActon("swipe", opts)
        return self._deal_result(result)

    def swipeOrientation(self, params):
        '''
        :Args:
            keyword    : swipeOrientation
            params     : direction(UP, DOWN, LEFT, RIGHT)
                        steps(int)
        '''
        opts = {
                "elementId": params["id"],
                "direction": params["direction"],
                "steps": (int)(math.ceil(params["duration"] * 20)) if params.get("duration") else self.steps,
                }
        result = self._uiautomator.sendActon("element:swipeOrientation", opts)
        return self._deal_result(result)

    def flick(self, params):
        opts = {
            "startX": params["x"],
            "startY": params["y"],
            "endX": params["x"],
            "endY": params["y"],
            "steps": (int)(math.ceil(params["duration"] * 20)) if params.get("duration") else self.steps
        }

        if params.get('id'):
            result = self._uiautomator.sendActon("element:swipe", opts)
        else:
            result = self._uiautomator.sendActon("swipe", opts)
        return self._deal_result(result)

    def scrollTo(self, params):
        """
        instead of the elementId as the element to be scrolled too,
        it's the scrollable view to swipe until the uiobject that has the text is found.
        """
        opts = {
            "elementId": params["id"],
            "text": params['text'],
            "direction": params["direction"],
        }
        result = self._uiautomator.sendActon("element:scrollTo", opts)
        return self._deal_result(result)

    def scrollOrientation(self, params):
        opts = {
            "elementId": params["id"],
            "direction": params["direction"],
            "steps" : params['steps']
        }
        result = self._uiautomator.sendActon("element:scrollOrientation", opts)
        return self._deal_result(result)

    def getAttribute(self, params):
        opts = {
            "elementId": params["id"],
            "attribute": params["attribute"]
        }
        result = self._uiautomator.sendActon("element:getAttribute", opts)
        if result:
            if result['status'] == 0: 
                if result[u'value'] == 'false':
                    result[u'value'] = False
                elif result[u'value'] == 'true':
                    result[u'value'] = True
                return result
        raise UiautomatorException(utils.formatExceptionStr(self._adb.serial, result))

    def setText(self, params):
        opts = {
            "elementId": params["id"] if params.get("id") else None,
            "text": params["text"],
            "replace": params["replace"]
        }
        if params.get("id"):
            result = self._uiautomator.sendActon("element:setText", opts)
        else:
            result = self._uiautomator.sendActon("setText", opts)
        return self._deal_result(result)

    def getText(self, params):
        result = self._uiautomator.sendActon("getText", params)
        return self._deal_result(result)
    

    def getSize(self, params):
        opts = {
            'elementId': params['id']
        }
        result = self._uiautomator.sendActon("element:getSize", opts)
        return self._deal_result(result)

    def getLocation(self, params):
        opts = {
            'elementId': params['id']
        }
        result = self._uiautomator.sendActon("element:getLocation", opts)
        return self._deal_result(result)

    def wake(self, params=None):
        result = self._uiautomator.sendActon("wake", params)
        return self._deal_result(result)
    
    def waitForIdle(self,params):
        result = self._uiautomator.sendActon("waitForIdle", params)
        return self._deal_result(result)

    def orientation(self, params):
        '''
        ROTATION_0(0), ROTATION_90(1), ROTATION_180(2), ROTATION_270(3)
        params: landscape,portrait
        '''
        opts = {}
        orientation = params.get('orientation')
        if orientation:
            opts['orientation'] = orientation
        result = self._uiautomator.sendActon("orientation", opts)
        return self._deal_result(result)
    
    def fileList(self,params):
        result = self._uiautomator.sendActon("fileList", params)
        return self._deal_result(result)

    def clearCache(self, params):
        result = self._uiautomator.sendActon("clearCache", params)
        return self._deal_result(result)
    
    def clearUiCache(self,params):
        result = self._uiautomator.sendActon("clearUiCache", params)
        return self._deal_result(result)
    
    def uiWatcher(self,params):
        result = self._uiautomator.sendActon("uiWatcher", params)
        return self._deal_result(result)

    #####################################################################################################
    ############################################   NEW PLUS    ##########################################
    #####################################################################################################

    def swipeDirection(self, params):
        opts = {
            "elementId": params["id"],
            "steps": (int)(math.ceil(params["duration"] * 20)) if params.get("duration") else 10,
            "direction": params["direction"],
        }
        result = self._uiautomator.sendActon("element:scrollOrientation", opts)
        return self._deal_result(result)

    def lock(self, params=None):
        result = self._uiautomator.sendActon("lock", params)
        time.sleep(1)
        return self._deal_result(result)
    
    def openQuickSetting(self,params):
        result = self._uiautomator.sendActon("openQuickSetting", params)
        return self._deal_result(result)
    
    def openRecentApp(self,params):
        result = self._uiautomator.sendActon("openRecentApp", params)
        return self._deal_result(result)
    
    def roatation(self, params):
        opts = {}
        roatation = params.get('status')
        if roatation == True:
            opts['status'] = 'freeze'    
        else:
            opts['status'] = 'unfreeze'
        result = self._uiautomator.sendActon("rotation", opts)
        return self._deal_result(result)
    
    def monitor(self, params):
        '''
        power monitor and adbd monitor  
        :Args:
        "power_status": Open if true, or shut down 
        "height": power monitor height threshold
        "low": power monitor low threshold
        "adbd_status": Open if true, or shut down
        "adbThreshold": adbd monitor duration time
        '''
        opts = {
            "power_status": params["power_status"],
            "height": params["high"] if params.get("high") else 40,
            "low": params["low"] if params.get("low") else 10,
            "adbd_status": params["adbd_status"],
            "adbdThreshold": params["adbdThreshold"] if params.get("adbdThreshold") else 20
        }
        result = self._uiautomator.sendActon("monitor", opts)
        return self._deal_result(result)
    
    def toast(self,params):
        result = self._uiautomator.sendActon("toast", params)
        return self._deal_result(result)
    
    def getParent(self,params):
        opts = {
            "elementId": params["id"],
        }
        result = self._uiautomator.sendActon("element:getParent", opts)
        return self._deal_result(result)
    
    def sendKeys(self, params):
        opts = {
            "text": params["text"],
            "replace": params["replace"]
        }
        result = self._uiautomator.sendActon("sendKeys", opts)
        return self._deal_result(result)
    
    def _deal_result(self,result):
        if result:
            if result['status'] == 0:
                return result
        raise UiautomatorException(utils.formatExceptionStr(self._adb.serial, result))
        
        
    ################################################################
    ####                          ADB                           ####
    ################################################################
    
    def takeScreenshot_adb(self, params):
        png = self._adb.takeScreenshot()
        if png is not None:
            return utils.formatReturnStr(0, png)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, "not png"))

    def startActivity(self, params):
        if type(params['package']) != str or type(params['activity']) != str:
            raise AdbException(str(utils.formatReturnStr(1, 'type of args must be string')))
        if params['package'] == '' or params['activity'] == '':
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'args cannot be null'))
        value = self._adb.startActivity('%s/%s' % (params['package'], params['activity']))
        if 'Error' not in str(value):
            return utils.formatReturnStr(0, value)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, value))
    
    def getCurrentApp(self, params=None):
        return utils.formatReturnStr(0, self._adb.getCurrentApp())

    def lanuchApp(self, params):  ################未能通过包名获取LAUNCHER活动，暂时需要传全值
        if type(params['apk']) != str or type(params['activity']) != str:
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'type of args must be string'))
        if params['apk'] == '' or params['activity'] == '':
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'args cannot be null'))
        value = self._adb.startActivity('%s/%s' % (params['apk'], params['activity']))
        if 'Error' not in str(value):
            return utils.formatReturnStr(0, value)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, value))

    def closeApp(self, params):
        if self._adb.forceStop(params['apk']) == []:
            return utils.formatReturnStr(0, 'close app success')
        else:
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'close app failed'))

    def lock_adb(self, params=None):
        self._adb.lock()
        time.sleep(1)
        if self.isLocked() == True:
            return utils.formatReturnStr(0, 'Lock Success')
        else:
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'Fail To Unlock'))

    def reset(self, params):
        value = self._adb.reset(params['pkg'])
        if 'Success' in str(value):
            return utils.formatReturnStr(0, value)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, value))

    def shake(self, params=None):
        self._adb.shake()
        raise AdbException(utils.formatExceptionStr(self._adb.serial, 'not implement'))

    def unlock(self, params=None):
        self._adb.unlock()
        if self.isLocked() == False:
            return utils.formatReturnStr(0, 'Unlock Success')
        else:
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'Fail To Unlock'))

    def isLocked(self):
        return self._adb.isScreenLocked()

    def input_adb(self, context):
        self._adb.type(context)

    def hideKeyboard(self, params=None):
        value = self._adb.hideKeyboard()
        if value == 'keyboard is not visible':
            return utils.formatReturnStr(0, value)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, value))

    def pullFile(self, params):
        values = self._adb.pull(params['srcfile'], params['tarfile'])
        for value in values:
            if '100%' in value:
                return utils.formatReturnStr(0, value)
        else:
            raise AdbException (utils.formatExceptionStr(self._adb.serial, values))

    def pullFolder(self, params):
        folderName = params['remotepath'].split("/")[-1]
        target_path = os.path.join(params['localpath'], folderName)
        if not os.path.exists(target_path):
            os.makedirs(target_path)
        values = self._adb.pull(params['remotepath'], target_path)
        for value in values:
            if '100%' in value:
                return utils.formatReturnStr(0, value)
        else:
            raise AdbException (utils.formatExceptionStr(self._adb.serial, values))

    def pushFile(self, params):
        values = self._adb.push(params['srcfile'], params['tarfile'])
        for value in values:
            if '100%' in value:
                return utils.formatReturnStr(0, value)
        else:
            raise AdbException (utils.formatExceptionStr(self._adb.serial, values))

    def background(self, params):
        if type(params['tms']) != int:
            raise AdbException(utils.formatExceptionStr(self._adb.serial, 'Canshu Wrong'))
        return utils.formatReturnStr(0, self._adb.background(params['tms']))

    def pressKeyCode_adb(self, keycode):
        self._adb.press(keycode)

    def longPressKeyCode_adb(self, keycode):
        self._adb.longPress(keycode)

    def installApp(self, params):
        result = self._adb.install(params['apk'],params['arg'])
        if 'Success' in str(result):
            return utils.formatReturnStr(0, result)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, result))
    
    def installRemoteApp(self, params):
        result = self._adb.installRemote(params['apk'])
        if 'Success' in str(result):
            return utils.formatReturnStr(0, result)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, result))
        
    def removeApp(self, params):
        result = self._adb.uninstallApk(params['apk'])
        if 'Success' in str(result):
            return utils.formatReturnStr(0, result)
        raise AdbException(utils.formatExceptionStr(self._adb.serial, result))
    
    def isAppInstalled(self, params):
        result = self._adb.isAppinstalled(params['apk'])
        if result == True:
            return utils.formatReturnStr(0, True)
        return utils.formatReturnStr(1, False)

    def doAdb(self, params):
        return utils.formatReturnStr(0, self._adb.exec_adb(params['command']))
    
    def isOnline(self,params=None):
        flag = self._adb.isOnline()
        return utils.formatReturnStr(0, flag)
      
    def isScreenLocked(self, params=None):
        flag = self._adb.isScreenLocked()
        return utils.formatReturnStr(0, flag)
    
    #####remote信息#################
    def remote_sim_info(self,parames=None):
        iccid = parames['imsi']
        data = ViewRemoteResource.getInstance().getSimInfo(iccid)
        if data['data']:
            return utils.formatReturnStr(0, data['data'])
        else:
            raise RemoteServiceException(utils.formatReturnStr(1, "not available data"))
